# Generated from CnsMRParser.g4 by ANTLR 4.9
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO


def serializedATN():
    with StringIO() as buf:
        buf.write("\3\u608b\ua72a\u8133\ub9ed\u417c\u3be7\u7786\u5964\3\u009a")
        buf.write("\u0467\4\2\t\2\4\3\t\3\4\4\t\4\4\5\t\5\4\6\t\6\4\7\t\7")
        buf.write("\4\b\t\b\4\t\t\t\4\n\t\n\4\13\t\13\4\f\t\f\4\r\t\r\4\16")
        buf.write("\t\16\4\17\t\17\4\20\t\20\4\21\t\21\4\22\t\22\4\23\t\23")
        buf.write("\4\24\t\24\4\25\t\25\4\26\t\26\4\27\t\27\4\30\t\30\4\31")
        buf.write("\t\31\4\32\t\32\4\33\t\33\4\34\t\34\4\35\t\35\4\36\t\36")
        buf.write("\4\37\t\37\4 \t \4!\t!\4\"\t\"\4#\t#\4$\t$\4%\t%\4&\t")
        buf.write("&\4\'\t\'\4(\t(\4)\t)\4*\t*\4+\t+\4,\t,\4-\t-\4.\t.\4")
        buf.write("/\t/\4\60\t\60\4\61\t\61\4\62\t\62\4\63\t\63\3\2\3\2\3")
        buf.write("\2\3\2\3\2\3\2\3\2\3\2\3\2\3\2\3\2\3\2\7\2s\n\2\f\2\16")
        buf.write("\2v\13\2\3\2\7\2y\n\2\f\2\16\2|\13\2\3\2\7\2\177\n\2\f")
        buf.write("\2\16\2\u0082\13\2\3\2\7\2\u0085\n\2\f\2\16\2\u0088\13")
        buf.write("\2\3\2\7\2\u008b\n\2\f\2\16\2\u008e\13\2\3\2\7\2\u0091")
        buf.write("\n\2\f\2\16\2\u0094\13\2\3\2\3\2\3\3\3\3\3\3\3\3\3\4\3")
        buf.write("\4\3\4\3\4\3\4\3\5\3\5\3\5\3\5\3\5\3\6\3\6\3\6\3\6\3\6")
        buf.write("\3\7\3\7\3\7\3\7\3\b\3\b\3\b\3\b\3\t\3\t\3\t\3\t\3\n\3")
        buf.write("\n\3\n\3\n\3\13\3\13\3\13\3\13\3\f\3\f\3\f\3\f\3\r\3\r")
        buf.write("\3\r\3\r\3\16\3\16\3\16\3\16\3\17\3\17\3\17\3\17\7\17")
        buf.write("\u00cf\n\17\f\17\16\17\u00d2\13\17\3\17\3\17\3\17\3\17")
        buf.write("\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17")
        buf.write("\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17")
        buf.write("\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\7\17\u00f7")
        buf.write("\n\17\f\17\16\17\u00fa\13\17\3\17\3\17\3\17\3\17\3\17")
        buf.write("\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17")
        buf.write("\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17")
        buf.write("\7\17\u0117\n\17\f\17\16\17\u011a\13\17\3\17\3\17\3\17")
        buf.write("\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17")
        buf.write("\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\7\17\u0132\n")
        buf.write("\17\f\17\16\17\u0135\13\17\3\17\3\17\3\17\3\17\5\17\u013b")
        buf.write("\n\17\3\20\3\20\3\20\3\20\3\20\3\20\3\20\3\20\3\20\3\20")
        buf.write("\7\20\u0147\n\20\f\20\16\20\u014a\13\20\3\21\3\21\3\21")
        buf.write("\3\21\3\21\3\21\3\21\3\21\3\21\3\21\5\21\u0156\n\21\3")
        buf.write("\22\7\22\u0159\n\22\f\22\16\22\u015c\13\22\3\22\3\22\3")
        buf.write("\22\3\22\3\22\3\22\3\22\3\22\3\22\3\22\3\22\3\22\3\22")
        buf.write("\3\22\5\22\u016c\n\22\3\23\3\23\3\23\3\23\3\23\3\23\3")
        buf.write("\23\3\23\3\23\3\23\3\24\3\24\3\24\3\24\5\24\u017c\n\24")
        buf.write("\3\25\3\25\3\25\3\25\3\25\3\25\5\25\u0184\n\25\3\26\3")
        buf.write("\26\3\26\3\26\3\26\3\26\3\26\3\26\5\26\u018e\n\26\3\26")
        buf.write("\3\26\5\26\u0192\n\26\3\26\3\26\3\26\3\26\3\26\5\26\u0199")
        buf.write("\n\26\3\26\3\26\3\26\5\26\u019e\n\26\5\26\u01a0\n\26\3")
        buf.write("\26\5\26\u01a3\n\26\3\27\7\27\u01a6\n\27\f\27\16\27\u01a9")
        buf.write("\13\27\3\27\3\27\3\27\3\27\3\27\3\27\3\27\3\27\3\27\3")
        buf.write("\27\3\27\3\27\3\27\3\27\3\27\3\27\3\27\3\27\3\27\3\27")
        buf.write("\5\27\u01bf\n\27\3\30\3\30\3\30\3\30\3\30\3\30\3\30\3")
        buf.write("\30\3\30\3\30\5\30\u01cb\n\30\3\31\7\31\u01ce\n\31\f\31")
        buf.write("\16\31\u01d1\13\31\3\31\3\31\3\31\3\31\3\31\3\31\3\31")
        buf.write("\3\31\3\31\3\31\3\31\3\31\3\31\3\31\5\31\u01e1\n\31\3")
        buf.write("\31\3\31\3\31\3\31\3\31\3\31\3\31\3\31\3\31\3\31\3\31")
        buf.write("\3\31\3\31\3\31\3\31\3\31\5\31\u01f3\n\31\3\31\5\31\u01f6")
        buf.write("\n\31\3\32\3\32\3\32\3\32\3\32\3\32\3\32\3\32\3\32\3\32")
        buf.write("\5\32\u0202\n\32\3\32\3\32\3\32\3\32\5\32\u0208\n\32\3")
        buf.write("\33\7\33\u020b\n\33\f\33\16\33\u020e\13\33\3\33\3\33\3")
        buf.write("\33\3\33\3\33\3\33\3\33\3\33\3\33\3\33\3\33\3\33\3\33")
        buf.write("\3\33\3\33\3\33\3\33\3\33\3\33\3\33\3\33\3\33\3\33\3\33")
        buf.write("\3\33\3\33\3\33\3\33\7\33\u022c\n\33\f\33\16\33\u022f")
        buf.write("\13\33\3\33\3\33\5\33\u0233\n\33\3\34\3\34\3\34\3\34\3")
        buf.write("\34\3\34\3\34\3\34\3\34\3\35\3\35\3\35\3\35\3\35\3\36")
        buf.write("\7\36\u0244\n\36\f\36\16\36\u0247\13\36\3\36\7\36\u024a")
        buf.write("\n\36\f\36\16\36\u024d\13\36\3\36\7\36\u0250\n\36\f\36")
        buf.write("\16\36\u0253\13\36\3\36\7\36\u0256\n\36\f\36\16\36\u0259")
        buf.write("\13\36\3\36\7\36\u025c\n\36\f\36\16\36\u025f\13\36\3\36")
        buf.write("\7\36\u0262\n\36\f\36\16\36\u0265\13\36\3\36\7\36\u0268")
        buf.write("\n\36\f\36\16\36\u026b\13\36\3\36\7\36\u026e\n\36\f\36")
        buf.write("\16\36\u0271\13\36\3\36\7\36\u0274\n\36\f\36\16\36\u0277")
        buf.write("\13\36\3\36\3\36\3\36\3\36\3\36\5\36\u027e\n\36\3\36\3")
        buf.write("\36\3\36\3\36\5\36\u0284\n\36\3\36\3\36\3\36\3\36\3\36")
        buf.write("\3\36\3\36\3\36\3\36\5\36\u028f\n\36\3\36\3\36\5\36\u0293")
        buf.write("\n\36\3\37\3\37\3\37\5\37\u0298\n\37\3\37\3\37\5\37\u029c")
        buf.write("\n\37\3 \3 \3 \3 \3!\3!\3!\3!\3!\3!\5!\u02a8\n!\3!\3!")
        buf.write("\3\"\3\"\3\"\3#\3#\3#\3$\3$\3$\3%\3%\3%\3&\3&\3&\3&\3")
        buf.write("&\3&\3&\3&\5&\u02c0\n&\3\'\3\'\3\'\3(\7(\u02c6\n(\f(\16")
        buf.write("(\u02c9\13(\3(\3(\3(\3(\3(\3(\3(\5(\u02d2\n(\3(\5(\u02d5")
        buf.write("\n(\3(\5(\u02d8\n(\3(\3(\3(\3(\3(\3(\3(\3(\3(\3(\3(\3")
        buf.write("(\3(\3(\3(\3(\3(\5(\u02eb\n(\3(\3(\3(\5(\u02f0\n(\3(\3")
        buf.write("(\3(\5(\u02f5\n(\3(\3(\3(\3(\3(\3(\3(\3(\3(\3(\5(\u0301")
        buf.write("\n(\3(\3(\3(\3(\3(\5(\u0308\n(\3(\5(\u030b\n(\3(\5(\u030e")
        buf.write("\n(\3(\5(\u0311\n(\3)\3)\3)\3)\3)\3)\3)\3)\3)\3)\5)\u031d")
        buf.write("\n)\3)\3)\3)\3)\3)\5)\u0324\n)\3)\3)\3)\3)\3)\5)\u032b")
        buf.write("\n)\3*\7*\u032e\n*\f*\16*\u0331\13*\3*\3*\3*\3*\3*\3*")
        buf.write("\3*\3*\3*\3*\3*\3*\3*\3*\3*\3*\3*\3*\3*\3*\3*\3*\5*\u0349")
        buf.write("\n*\3+\3+\3+\3+\3+\3+\3+\3+\3+\3+\3,\7,\u0356\n,\f,\16")
        buf.write(",\u0359\13,\3,\3,\3,\3,\3,\3,\3,\3,\3,\3,\3,\3,\3,\3,")
        buf.write("\3,\3,\3,\3,\3,\3,\3,\3,\3,\3,\5,\u0373\n,\3-\3-\3-\3")
        buf.write("-\3-\3-\3-\3-\3-\3-\3-\3-\3.\7.\u0382\n.\f.\16.\u0385")
        buf.write("\13.\3.\3.\3.\3.\3.\3.\3.\3.\3.\3.\3.\3.\3.\3.\3.\3.\3")
        buf.write(".\3.\3.\3.\3.\3.\3.\3.\3.\3.\3.\3.\3.\5.\u03a4\n.\3.\3")
        buf.write(".\3.\3.\3.\3.\5.\u03ac\n.\3/\3/\3/\3/\3/\3/\3/\3/\3/\3")
        buf.write("/\3/\3/\3/\5/\u03bb\n/\3\60\3\60\3\60\3\60\3\61\3\61\3")
        buf.write("\61\7\61\u03c4\n\61\f\61\16\61\u03c7\13\61\3\62\3\62\3")
        buf.write("\62\7\62\u03cc\n\62\f\62\16\62\u03cf\13\62\3\63\3\63\3")
        buf.write("\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\5\63")
        buf.write("\u03dd\n\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3")
        buf.write("\63\3\63\3\63\3\63\3\63\3\63\5\63\u03ed\n\63\5\63\u03ef")
        buf.write("\n\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63")
        buf.write("\3\63\3\63\3\63\3\63\3\63\3\63\5\63\u0401\n\63\5\63\u0403")
        buf.write("\n\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\5\63\u040c\n")
        buf.write("\63\3\63\3\63\5\63\u0410\n\63\3\63\3\63\3\63\3\63\3\63")
        buf.write("\3\63\3\63\3\63\3\63\3\63\5\63\u041c\n\63\3\63\3\63\3")
        buf.write("\63\5\63\u0421\n\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63")
        buf.write("\3\63\3\63\3\63\3\63\5\63\u042e\n\63\5\63\u0430\n\63\3")
        buf.write("\63\3\63\3\63\3\63\3\63\5\63\u0437\n\63\5\63\u0439\n\63")
        buf.write("\3\63\3\63\3\63\3\63\3\63\5\63\u0440\n\63\3\63\3\63\3")
        buf.write("\63\5\63\u0445\n\63\5\63\u0447\n\63\3\63\3\63\3\63\3\63")
        buf.write("\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63")
        buf.write("\3\63\3\63\5\63\u045a\n\63\3\63\3\63\3\63\3\63\3\63\3")
        buf.write("\63\7\63\u0462\n\63\f\63\16\63\u0465\13\63\3\63\5\u00f8")
        buf.write("\u0118\u0133\3d\64\2\4\6\b\n\f\16\20\22\24\26\30\32\34")
        buf.write("\36 \"$&(*,.\60\62\64\668:<>@BDFHJLNPRTVXZ\\^`bd\2\5\4")
        buf.write("\2\u0083\u0083\u0085\u0085\3\2\u0087\u0088\4\2\u0083\u0083")
        buf.write("\u0089\u0089\2\u052a\2t\3\2\2\2\4\u0097\3\2\2\2\6\u009b")
        buf.write("\3\2\2\2\b\u00a0\3\2\2\2\n\u00a5\3\2\2\2\f\u00aa\3\2\2")
        buf.write("\2\16\u00ae\3\2\2\2\20\u00b2\3\2\2\2\22\u00b6\3\2\2\2")
        buf.write("\24\u00ba\3\2\2\2\26\u00be\3\2\2\2\30\u00c2\3\2\2\2\32")
        buf.write("\u00c6\3\2\2\2\34\u013a\3\2\2\2\36\u013c\3\2\2\2 \u0155")
        buf.write("\3\2\2\2\"\u016b\3\2\2\2$\u016d\3\2\2\2&\u017b\3\2\2\2")
        buf.write("(\u0183\3\2\2\2*\u01a2\3\2\2\2,\u01be\3\2\2\2.\u01c0\3")
        buf.write("\2\2\2\60\u01f5\3\2\2\2\62\u01f7\3\2\2\2\64\u0232\3\2")
        buf.write("\2\2\66\u0234\3\2\2\28\u023d\3\2\2\2:\u0292\3\2\2\2<\u0294")
        buf.write("\3\2\2\2>\u029d\3\2\2\2@\u02a1\3\2\2\2B\u02ab\3\2\2\2")
        buf.write("D\u02ae\3\2\2\2F\u02b1\3\2\2\2H\u02b4\3\2\2\2J\u02b7\3")
        buf.write("\2\2\2L\u02c1\3\2\2\2N\u0310\3\2\2\2P\u0312\3\2\2\2R\u0348")
        buf.write("\3\2\2\2T\u034a\3\2\2\2V\u0372\3\2\2\2X\u0374\3\2\2\2")
        buf.write("Z\u03ab\3\2\2\2\\\u03ad\3\2\2\2^\u03bc\3\2\2\2`\u03c0")
        buf.write("\3\2\2\2b\u03c8\3\2\2\2d\u0459\3\2\2\2fs\5\4\3\2gs\5\6")
        buf.write("\4\2hs\5\b\5\2is\5\n\6\2js\5\f\7\2ks\5\16\b\2ls\5\20\t")
        buf.write("\2ms\5\22\n\2ns\5\24\13\2os\5\26\f\2ps\5\30\r\2qs\5\32")
        buf.write("\16\2rf\3\2\2\2rg\3\2\2\2rh\3\2\2\2ri\3\2\2\2rj\3\2\2")
        buf.write("\2rk\3\2\2\2rl\3\2\2\2rm\3\2\2\2rn\3\2\2\2ro\3\2\2\2r")
        buf.write("p\3\2\2\2rq\3\2\2\2sv\3\2\2\2tr\3\2\2\2tu\3\2\2\2uz\3")
        buf.write("\2\2\2vt\3\2\2\2wy\5\36\20\2xw\3\2\2\2y|\3\2\2\2zx\3\2")
        buf.write("\2\2z{\3\2\2\2{\u0080\3\2\2\2|z\3\2\2\2}\177\5$\23\2~")
        buf.write("}\3\2\2\2\177\u0082\3\2\2\2\u0080~\3\2\2\2\u0080\u0081")
        buf.write("\3\2\2\2\u0081\u0086\3\2\2\2\u0082\u0080\3\2\2\2\u0083")
        buf.write("\u0085\5.\30\2\u0084\u0083\3\2\2\2\u0085\u0088\3\2\2\2")
        buf.write("\u0086\u0084\3\2\2\2\u0086\u0087\3\2\2\2\u0087\u008c\3")
        buf.write("\2\2\2\u0088\u0086\3\2\2\2\u0089\u008b\5&\24\2\u008a\u0089")
        buf.write("\3\2\2\2\u008b\u008e\3\2\2\2\u008c\u008a\3\2\2\2\u008c")
        buf.write("\u008d\3\2\2\2\u008d\u0092\3\2\2\2\u008e\u008c\3\2\2\2")
        buf.write("\u008f\u0091\5\62\32\2\u0090\u008f\3\2\2\2\u0091\u0094")
        buf.write("\3\2\2\2\u0092\u0090\3\2\2\2\u0092\u0093\3\2\2\2\u0093")
        buf.write("\u0095\3\2\2\2\u0094\u0092\3\2\2\2\u0095\u0096\7\2\2\3")
        buf.write("\u0096\3\3\2\2\2\u0097\u0098\7\5\2\2\u0098\u0099\5\34")
        buf.write("\17\2\u0099\u009a\7\4\2\2\u009a\5\3\2\2\2\u009b\u009c")
        buf.write("\7\60\2\2\u009c\u009d\7\61\2\2\u009d\u009e\5\"\22\2\u009e")
        buf.write("\u009f\7\4\2\2\u009f\7\3\2\2\2\u00a0\u00a1\7\60\2\2\u00a1")
        buf.write("\u00a2\7\64\2\2\u00a2\u00a3\5&\24\2\u00a3\u00a4\7\4\2")
        buf.write("\2\u00a4\t\3\2\2\2\u00a5\u00a6\7\60\2\2\u00a6\u00a7\7")
        buf.write("8\2\2\u00a7\u00a8\5*\26\2\u00a8\u00a9\7\4\2\2\u00a9\13")
        buf.write("\3\2\2\2\u00aa\u00ab\7;\2\2\u00ab\u00ac\5,\27\2\u00ac")
        buf.write("\u00ad\7\4\2\2\u00ad\r\3\2\2\2\u00ae\u00af\7?\2\2\u00af")
        buf.write("\u00b0\5\60\31\2\u00b0\u00b1\7\4\2\2\u00b1\17\3\2\2\2")
        buf.write("\u00b2\u00b3\7A\2\2\u00b3\u00b4\5\64\33\2\u00b4\u00b5")
        buf.write("\7\4\2\2\u00b5\21\3\2\2\2\u00b6\u00b7\7G\2\2\u00b7\u00b8")
        buf.write("\5:\36\2\u00b8\u00b9\7\4\2\2\u00b9\23\3\2\2\2\u00ba\u00bb")
        buf.write("\7Q\2\2\u00bb\u00bc\5N(\2\u00bc\u00bd\7\4\2\2\u00bd\25")
        buf.write("\3\2\2\2\u00be\u00bf\7V\2\2\u00bf\u00c0\5R*\2\u00c0\u00c1")
        buf.write("\7\4\2\2\u00c1\27\3\2\2\2\u00c2\u00c3\7W\2\2\u00c3\u00c4")
        buf.write("\5V,\2\u00c4\u00c5\7\4\2\2\u00c5\31\3\2\2\2\u00c6\u00c7")
        buf.write("\7X\2\2\u00c7\u00c8\5Z.\2\u00c8\u00c9\7\4\2\2\u00c9\33")
        buf.write("\3\2\2\2\u00ca\u00cb\7\6\2\2\u00cb\u00cc\7\u008d\2\2\u00cc")
        buf.write("\u013b\7%\2\2\u00cd\u00cf\5\36\20\2\u00ce\u00cd\3\2\2")
        buf.write("\2\u00cf\u00d2\3\2\2\2\u00d0\u00ce\3\2\2\2\u00d0\u00d1")
        buf.write("\3\2\2\2\u00d1\u013b\3\2\2\2\u00d2\u00d0\3\2\2\2\u00d3")
        buf.write("\u00d4\7\b\2\2\u00d4\u00d5\7\u0088\2\2\u00d5\u013b\7\u0085")
        buf.write("\2\2\u00d6\u00d7\7\t\2\2\u00d7\u00d8\7\u0088\2\2\u00d8")
        buf.write("\u013b\7*\2\2\u00d9\u00da\7\n\2\2\u00da\u00db\7\u0088")
        buf.write("\2\2\u00db\u013b\7\u0085\2\2\u00dc\u00dd\7\13\2\2\u00dd")
        buf.write("\u00de\7\u008d\2\2\u00de\u013b\7\u0085\2\2\u00df\u00e0")
        buf.write("\7\f\2\2\u00e0\u00e1\7\u008d\2\2\u00e1\u013b\7\u0087\2")
        buf.write("\2\u00e2\u00e3\7\r\2\2\u00e3\u013b\7\u0087\2\2\u00e4\u00e5")
        buf.write("\7\16\2\2\u00e5\u00e6\7\u008d\2\2\u00e6\u013b\7\u0083")
        buf.write("\2\2\u00e7\u00e8\7\17\2\2\u00e8\u013b\7&\2\2\u00e9\u00ea")
        buf.write("\7\17\2\2\u00ea\u00eb\7\'\2\2\u00eb\u00ec\7(\2\2\u00ec")
        buf.write("\u00ed\7\u008d\2\2\u00ed\u00ee\7\u0085\2\2\u00ee\u00ef")
        buf.write("\7)\2\2\u00ef\u00f0\7\u008d\2\2\u00f0\u013b\7\u0085\2")
        buf.write("\2\u00f1\u00f2\7\20\2\2\u00f2\u00f3\7\u0087\2\2\u00f3")
        buf.write("\u00f4\7\u0087\2\2\u00f4\u013b\7\u0085\2\2\u00f5\u00f7")
        buf.write("\7\21\2\2\u00f6\u00f5\3\2\2\2\u00f7\u00fa\3\2\2\2\u00f8")
        buf.write("\u00f9\3\2\2\2\u00f8\u00f6\3\2\2\2\u00f9\u00fb\3\2\2\2")
        buf.write("\u00fa\u00f8\3\2\2\2\u00fb\u013b\7\4\2\2\u00fc\u00fd\7")
        buf.write("\22\2\2\u00fd\u00fe\7\u0088\2\2\u00fe\u013b\7\u0083\2")
        buf.write("\2\u00ff\u0100\7\23\2\2\u0100\u0101\7\u0088\2\2\u0101")
        buf.write("\u013b\7\u0083\2\2\u0102\u0103\7\24\2\2\u0103\u0104\7")
        buf.write("\u008d\2\2\u0104\u013b\7\u0083\2\2\u0105\u013b\7\25\2")
        buf.write("\2\u0106\u0107\7\26\2\2\u0107\u0108\7\u008d\2\2\u0108")
        buf.write("\u013b\7\u0083\2\2\u0109\u010a\7\27\2\2\u010a\u010b\7")
        buf.write("\u0088\2\2\u010b\u013b\7+\2\2\u010c\u010d\7\30\2\2\u010d")
        buf.write("\u010e\5 \21\2\u010e\u010f\7\4\2\2\u010f\u013b\3\2\2\2")
        buf.write("\u0110\u0111\7\31\2\2\u0111\u0112\7\33\2\2\u0112\u0113")
        buf.write("\7\u008d\2\2\u0113\u013b\7\u0085\2\2\u0114\u0118\7\32")
        buf.write("\2\2\u0115\u0117\7\u0087\2\2\u0116\u0115\3\2\2\2\u0117")
        buf.write("\u011a\3\2\2\2\u0118\u0119\3\2\2\2\u0118\u0116\3\2\2\2")
        buf.write("\u0119\u011b\3\2\2\2\u011a\u0118\3\2\2\2\u011b\u013b\7")
        buf.write("\4\2\2\u011c\u013b\7\34\2\2\u011d\u011e\7\35\2\2\u011e")
        buf.write("\u011f\7\u0088\2\2\u011f\u013b\7\u0085\2\2\u0120\u0121")
        buf.write("\7\36\2\2\u0121\u0122\7\u0088\2\2\u0122\u013b\7\u0085")
        buf.write("\2\2\u0123\u0124\7\37\2\2\u0124\u0125\7\u0088\2\2\u0125")
        buf.write("\u013b\7\u0085\2\2\u0126\u0127\7 \2\2\u0127\u0128\7\u0088")
        buf.write("\2\2\u0128\u013b\7\u0085\2\2\u0129\u012a\7!\2\2\u012a")
        buf.write("\u012b\7\u0088\2\2\u012b\u013b\7\u0085\2\2\u012c\u012d")
        buf.write("\7\"\2\2\u012d\u012e\7\u0088\2\2\u012e\u013b\7\u0085\2")
        buf.write("\2\u012f\u0133\7#\2\2\u0130\u0132\7\u0087\2\2\u0131\u0130")
        buf.write("\3\2\2\2\u0132\u0135\3\2\2\2\u0133\u0134\3\2\2\2\u0133")
        buf.write("\u0131\3\2\2\2\u0134\u0136\3\2\2\2\u0135\u0133\3\2\2\2")
        buf.write("\u0136\u013b\7\4\2\2\u0137\u0138\7$\2\2\u0138\u0139\7")
        buf.write("\u008d\2\2\u0139\u013b\7\u0085\2\2\u013a\u00ca\3\2\2\2")
        buf.write("\u013a\u00d0\3\2\2\2\u013a\u00d3\3\2\2\2\u013a\u00d6\3")
        buf.write("\2\2\2\u013a\u00d9\3\2\2\2\u013a\u00dc\3\2\2\2\u013a\u00df")
        buf.write("\3\2\2\2\u013a\u00e2\3\2\2\2\u013a\u00e4\3\2\2\2\u013a")
        buf.write("\u00e7\3\2\2\2\u013a\u00e9\3\2\2\2\u013a\u00f1\3\2\2\2")
        buf.write("\u013a\u00f8\3\2\2\2\u013a\u00fc\3\2\2\2\u013a\u00ff\3")
        buf.write("\2\2\2\u013a\u0102\3\2\2\2\u013a\u0105\3\2\2\2\u013a\u0106")
        buf.write("\3\2\2\2\u013a\u0109\3\2\2\2\u013a\u010c\3\2\2\2\u013a")
        buf.write("\u0110\3\2\2\2\u013a\u0114\3\2\2\2\u013a\u011c\3\2\2\2")
        buf.write("\u013a\u011d\3\2\2\2\u013a\u0120\3\2\2\2\u013a\u0123\3")
        buf.write("\2\2\2\u013a\u0126\3\2\2\2\u013a\u0129\3\2\2\2\u013a\u012c")
        buf.write("\3\2\2\2\u013a\u012f\3\2\2\2\u013a\u0137\3\2\2\2\u013b")
        buf.write("\35\3\2\2\2\u013c\u013d\7\7\2\2\u013d\u013e\5^\60\2\u013e")
        buf.write("\u013f\5^\60\2\u013f\u0140\t\2\2\2\u0140\u0141\7\u0085")
        buf.write("\2\2\u0141\u0148\7\u0085\2\2\u0142\u0143\7~\2\2\u0143")
        buf.write("\u0144\5^\60\2\u0144\u0145\5^\60\2\u0145\u0147\3\2\2\2")
        buf.write("\u0146\u0142\3\2\2\2\u0147\u014a\3\2\2\2\u0148\u0146\3")
        buf.write("\2\2\2\u0148\u0149\3\2\2\2\u0149\37\3\2\2\2\u014a\u0148")
        buf.write("\3\2\2\2\u014b\u014c\7,\2\2\u014c\u014d\7\u008d\2\2\u014d")
        buf.write("\u0156\7\u0085\2\2\u014e\u014f\7-\2\2\u014f\u0150\7\u008d")
        buf.write("\2\2\u0150\u0156\7\u0085\2\2\u0151\u0152\7.\2\2\u0152")
        buf.write("\u0156\5^\60\2\u0153\u0154\7/\2\2\u0154\u0156\5^\60\2")
        buf.write("\u0155\u014b\3\2\2\2\u0155\u014e\3\2\2\2\u0155\u0151\3")
        buf.write("\2\2\2\u0155\u0153\3\2\2\2\u0156!\3\2\2\2\u0157\u0159")
        buf.write("\5$\23\2\u0158\u0157\3\2\2\2\u0159\u015c\3\2\2\2\u015a")
        buf.write("\u0158\3\2\2\2\u015a\u015b\3\2\2\2\u015b\u016c\3\2\2\2")
        buf.write("\u015c\u015a\3\2\2\2\u015d\u015e\7\16\2\2\u015e\u015f")
        buf.write("\7\u008d\2\2\u015f\u016c\7\u0083\2\2\u0160\u0161\7\62")
        buf.write("\2\2\u0161\u0162\7\u008d\2\2\u0162\u016c\7\u0083\2\2\u0163")
        buf.write("\u0164\7\26\2\2\u0164\u0165\7\u008d\2\2\u0165\u016c\7")
        buf.write("\u0083\2\2\u0166\u016c\7\34\2\2\u0167\u0168\7\36\2\2\u0168")
        buf.write("\u0169\7\u008d\2\2\u0169\u016c\7\u0085\2\2\u016a\u016c")
        buf.write("\7\63\2\2\u016b\u015a\3\2\2\2\u016b\u015d\3\2\2\2\u016b")
        buf.write("\u0160\3\2\2\2\u016b\u0163\3\2\2\2\u016b\u0166\3\2\2\2")
        buf.write("\u016b\u0167\3\2\2\2\u016b\u016a\3\2\2\2\u016c#\3\2\2")
        buf.write("\2\u016d\u016e\7\7\2\2\u016e\u016f\5^\60\2\u016f\u0170")
        buf.write("\5^\60\2\u0170\u0171\5^\60\2\u0171\u0172\5^\60\2\u0172")
        buf.write("\u0173\t\2\2\2\u0173\u0174\7\u0085\2\2\u0174\u0175\7\u0085")
        buf.write("\2\2\u0175\u0176\7\u0083\2\2\u0176%\3\2\2\2\u0177\u0178")
        buf.write("\7\65\2\2\u0178\u017c\5(\25\2\u0179\u017c\7&\2\2\u017a")
        buf.write("\u017c\7\63\2\2\u017b\u0177\3\2\2\2\u017b\u0179\3\2\2")
        buf.write("\2\u017b\u017a\3\2\2\2\u017c\'\3\2\2\2\u017d\u017e\7\66")
        buf.write("\2\2\u017e\u017f\7\u008d\2\2\u017f\u0184\5^\60\2\u0180")
        buf.write("\u0181\7\67\2\2\u0181\u0182\7\u008d\2\2\u0182\u0184\t")
        buf.write("\2\2\2\u0183\u017d\3\2\2\2\u0183\u0180\3\2\2\2\u0184)")
        buf.write("\3\2\2\2\u0185\u0186\79\2\2\u0186\u0187\7\u008d\2\2\u0187")
        buf.write("\u01a3\7\u0083\2\2\u0188\u0189\7:\2\2\u0189\u018a\7\u008d")
        buf.write("\2\2\u018a\u019f\7\u008a\2\2\u018b\u018d\7\u0085\2\2\u018c")
        buf.write("\u018e\7\u0081\2\2\u018d\u018c\3\2\2\2\u018d\u018e\3\2")
        buf.write("\2\2\u018e\u018f\3\2\2\2\u018f\u0191\7\u0085\2\2\u0190")
        buf.write("\u0192\7\u0081\2\2\u0191\u0190\3\2\2\2\u0191\u0192\3\2")
        buf.write("\2\2\u0192\u0193\3\2\2\2\u0193\u01a0\7\u0085\2\2\u0194")
        buf.write("\u0195\7|\2\2\u0195\u0196\7\u008d\2\2\u0196\u0198\5^\60")
        buf.write("\2\u0197\u0199\7\u0081\2\2\u0198\u0197\3\2\2\2\u0198\u0199")
        buf.write("\3\2\2\2\u0199\u019d\3\2\2\2\u019a\u019b\7}\2\2\u019b")
        buf.write("\u019c\7\u008d\2\2\u019c\u019e\5^\60\2\u019d\u019a\3\2")
        buf.write("\2\2\u019d\u019e\3\2\2\2\u019e\u01a0\3\2\2\2\u019f\u018b")
        buf.write("\3\2\2\2\u019f\u0194\3\2\2\2\u01a0\u01a1\3\2\2\2\u01a1")
        buf.write("\u01a3\7\u008b\2\2\u01a2\u0185\3\2\2\2\u01a2\u0188\3\2")
        buf.write("\2\2\u01a3+\3\2\2\2\u01a4\u01a6\5.\30\2\u01a5\u01a4\3")
        buf.write("\2\2\2\u01a6\u01a9\3\2\2\2\u01a7\u01a5\3\2\2\2\u01a7\u01a8")
        buf.write("\3\2\2\2\u01a8\u01bf\3\2\2\2\u01a9\u01a7\3\2\2\2\u01aa")
        buf.write("\u01ab\7\f\2\2\u01ab\u01ac\7\u008d\2\2\u01ac\u01bf\7\u0087")
        buf.write("\2\2\u01ad\u01ae\7<\2\2\u01ae\u01af\7\u0085\2\2\u01af")
        buf.write("\u01b0\7\u0085\2\2\u01b0\u01bf\7\u0085\2\2\u01b1\u01b2")
        buf.write("\7=\2\2\u01b2\u01b3\7\u008d\2\2\u01b3\u01bf\7\u0085\2")
        buf.write("\2\u01b4\u01b5\7\24\2\2\u01b5\u01b6\7\u008d\2\2\u01b6")
        buf.write("\u01bf\7\u0083\2\2\u01b7\u01b8\7\27\2\2\u01b8\u01b9\7")
        buf.write("\u008d\2\2\u01b9\u01bf\7>\2\2\u01ba\u01bb\7\31\2\2\u01bb")
        buf.write("\u01bc\7\33\2\2\u01bc\u01bf\7\u0085\2\2\u01bd\u01bf\7")
        buf.write("\34\2\2\u01be\u01a7\3\2\2\2\u01be\u01aa\3\2\2\2\u01be")
        buf.write("\u01ad\3\2\2\2\u01be\u01b1\3\2\2\2\u01be\u01b4\3\2\2\2")
        buf.write("\u01be\u01b7\3\2\2\2\u01be\u01ba\3\2\2\2\u01be\u01bd\3")
        buf.write("\2\2\2\u01bf-\3\2\2\2\u01c0\u01c1\7\7\2\2\u01c1\u01c2")
        buf.write("\5^\60\2\u01c2\u01c3\5^\60\2\u01c3\u01c4\5^\60\2\u01c4")
        buf.write("\u01c5\5^\60\2\u01c5\u01c6\5^\60\2\u01c6\u01c7\5^\60\2")
        buf.write("\u01c7\u01c8\t\2\2\2\u01c8\u01ca\7\u0085\2\2\u01c9\u01cb")
        buf.write("\7\u0085\2\2\u01ca\u01c9\3\2\2\2\u01ca\u01cb\3\2\2\2\u01cb")
        buf.write("/\3\2\2\2\u01cc\u01ce\5\62\32\2\u01cd\u01cc\3\2\2\2\u01ce")
        buf.write("\u01d1\3\2\2\2\u01cf\u01cd\3\2\2\2\u01cf\u01d0\3\2\2\2")
        buf.write("\u01d0\u01f6\3\2\2\2\u01d1\u01cf\3\2\2\2\u01d2\u01d3\7")
        buf.write("\f\2\2\u01d3\u01d4\7\u008d\2\2\u01d4\u01f6\7\u0087\2\2")
        buf.write("\u01d5\u01d6\7<\2\2\u01d6\u01d7\7\u0085\2\2\u01d7\u01d8")
        buf.write("\7\u0085\2\2\u01d8\u01d9\7\u0085\2\2\u01d9\u01f6\7\u0085")
        buf.write("\2\2\u01da\u01db\7\16\2\2\u01db\u01dc\7\u008d\2\2\u01dc")
        buf.write("\u01f6\7\u0083\2\2\u01dd\u01de\7=\2\2\u01de\u01e0\7\u0085")
        buf.write("\2\2\u01df\u01e1\7\u0085\2\2\u01e0\u01df\3\2\2\2\u01e0")
        buf.write("\u01e1\3\2\2\2\u01e1\u01f6\3\2\2\2\u01e2\u01e3\7\24\2")
        buf.write("\2\u01e3\u01e4\7\u008d\2\2\u01e4\u01f6\7\u0083\2\2\u01e5")
        buf.write("\u01e6\7\26\2\2\u01e6\u01e7\7\u008d\2\2\u01e7\u01f6\7")
        buf.write("\u0083\2\2\u01e8\u01e9\7\27\2\2\u01e9\u01ea\7\u008d\2")
        buf.write("\2\u01ea\u01f6\7@\2\2\u01eb\u01ec\7\31\2\2\u01ec\u01ed")
        buf.write("\7\33\2\2\u01ed\u01f2\7\u0085\2\2\u01ee\u01f3\7[\2\2\u01ef")
        buf.write("\u01f0\7\f\2\2\u01f0\u01f1\7\u008d\2\2\u01f1\u01f3\7\u0087")
        buf.write("\2\2\u01f2\u01ee\3\2\2\2\u01f2\u01ef\3\2\2\2\u01f3\u01f6")
        buf.write("\3\2\2\2\u01f4\u01f6\7\34\2\2\u01f5\u01cf\3\2\2\2\u01f5")
        buf.write("\u01d2\3\2\2\2\u01f5\u01d5\3\2\2\2\u01f5\u01da\3\2\2\2")
        buf.write("\u01f5\u01dd\3\2\2\2\u01f5\u01e2\3\2\2\2\u01f5\u01e5\3")
        buf.write("\2\2\2\u01f5\u01e8\3\2\2\2\u01f5\u01eb\3\2\2\2\u01f5\u01f4")
        buf.write("\3\2\2\2\u01f6\61\3\2\2\2\u01f7\u01f8\7\7\2\2\u01f8\u01f9")
        buf.write("\5^\60\2\u01f9\u01fa\5^\60\2\u01fa\u01fb\5^\60\2\u01fb")
        buf.write("\u0201\5^\60\2\u01fc\u01fd\5^\60\2\u01fd\u01fe\5^\60\2")
        buf.write("\u01fe\u01ff\5^\60\2\u01ff\u0200\5^\60\2\u0200\u0202\3")
        buf.write("\2\2\2\u0201\u01fc\3\2\2\2\u0201\u0202\3\2\2\2\u0202\u0203")
        buf.write("\3\2\2\2\u0203\u0204\7\u0085\2\2\u0204\u0207\7\u0085\2")
        buf.write("\2\u0205\u0206\7\u0085\2\2\u0206\u0208\7\u0085\2\2\u0207")
        buf.write("\u0205\3\2\2\2\u0207\u0208\3\2\2\2\u0208\63\3\2\2\2\u0209")
        buf.write("\u020b\5\66\34\2\u020a\u0209\3\2\2\2\u020b\u020e\3\2\2")
        buf.write("\2\u020c\u020a\3\2\2\2\u020c\u020d\3\2\2\2\u020d\u0233")
        buf.write("\3\2\2\2\u020e\u020c\3\2\2\2\u020f\u0210\7\f\2\2\u0210")
        buf.write("\u0211\7\u008d\2\2\u0211\u0233\7\u0087\2\2\u0212\u0213")
        buf.write("\7B\2\2\u0213\u0214\7\u0083\2\2\u0214\u0215\7\u0083\2")
        buf.write("\2\u0215\u0216\7\u0085\2\2\u0216\u0217\7\u0085\2\2\u0217")
        buf.write("\u0233\7\u0085\2\2\u0218\u0219\7=\2\2\u0219\u021a\7\u008d")
        buf.write("\2\2\u021a\u0233\7\u0085\2\2\u021b\u021c\7\24\2\2\u021c")
        buf.write("\u021d\7\u008d\2\2\u021d\u0233\7\u0083\2\2\u021e\u021f")
        buf.write("\7C\2\2\u021f\u0220\7\u008d\2\2\u0220\u0233\7\u0085\2")
        buf.write("\2\u0221\u0222\7D\2\2\u0222\u0223\7\u008d\2\2\u0223\u0233")
        buf.write("\7\u0085\2\2\u0224\u0225\7\27\2\2\u0225\u0226\7\u008d")
        buf.write("\2\2\u0226\u0233\7>\2\2\u0227\u0228\7\31\2\2\u0228\u0229")
        buf.write("\7\33\2\2\u0229\u0233\7\u0085\2\2\u022a\u022c\58\35\2")
        buf.write("\u022b\u022a\3\2\2\2\u022c\u022f\3\2\2\2\u022d\u022b\3")
        buf.write("\2\2\2\u022d\u022e\3\2\2\2\u022e\u0233\3\2\2\2\u022f\u022d")
        buf.write("\3\2\2\2\u0230\u0233\7\34\2\2\u0231\u0233\7F\2\2\u0232")
        buf.write("\u020c\3\2\2\2\u0232\u020f\3\2\2\2\u0232\u0212\3\2\2\2")
        buf.write("\u0232\u0218\3\2\2\2\u0232\u021b\3\2\2\2\u0232\u021e\3")
        buf.write("\2\2\2\u0232\u0221\3\2\2\2\u0232\u0224\3\2\2\2\u0232\u0227")
        buf.write("\3\2\2\2\u0232\u022d\3\2\2\2\u0232\u0230\3\2\2\2\u0232")
        buf.write("\u0231\3\2\2\2\u0233\65\3\2\2\2\u0234\u0235\7\7\2\2\u0235")
        buf.write("\u0236\5^\60\2\u0236\u0237\5^\60\2\u0237\u0238\5^\60\2")
        buf.write("\u0238\u0239\5^\60\2\u0239\u023a\5^\60\2\u023a\u023b\7")
        buf.write("\u0085\2\2\u023b\u023c\7\u0085\2\2\u023c\67\3\2\2\2\u023d")
        buf.write("\u023e\7E\2\2\u023e\u023f\5^\60\2\u023f\u0240\7\u0085")
        buf.write("\2\2\u0240\u0241\7\u0085\2\2\u02419\3\2\2\2\u0242\u0244")
        buf.write("\5<\37\2\u0243\u0242\3\2\2\2\u0244\u0247\3\2\2\2\u0245")
        buf.write("\u0243\3\2\2\2\u0245\u0246\3\2\2\2\u0246\u0293\3\2\2\2")
        buf.write("\u0247\u0245\3\2\2\2\u0248\u024a\5> \2\u0249\u0248\3\2")
        buf.write("\2\2\u024a\u024d\3\2\2\2\u024b\u0249\3\2\2\2\u024b\u024c")
        buf.write("\3\2\2\2\u024c\u0293\3\2\2\2\u024d\u024b\3\2\2\2\u024e")
        buf.write("\u0250\5@!\2\u024f\u024e\3\2\2\2\u0250\u0253\3\2\2\2\u0251")
        buf.write("\u024f\3\2\2\2\u0251\u0252\3\2\2\2\u0252\u0293\3\2\2\2")
        buf.write("\u0253\u0251\3\2\2\2\u0254\u0256\5B\"\2\u0255\u0254\3")
        buf.write("\2\2\2\u0256\u0259\3\2\2\2\u0257\u0255\3\2\2\2\u0257\u0258")
        buf.write("\3\2\2\2\u0258\u0293\3\2\2\2\u0259\u0257\3\2\2\2\u025a")
        buf.write("\u025c\5D#\2\u025b\u025a\3\2\2\2\u025c\u025f\3\2\2\2\u025d")
        buf.write("\u025b\3\2\2\2\u025d\u025e\3\2\2\2\u025e\u0293\3\2\2\2")
        buf.write("\u025f\u025d\3\2\2\2\u0260\u0262\5F$\2\u0261\u0260\3\2")
        buf.write("\2\2\u0262\u0265\3\2\2\2\u0263\u0261\3\2\2\2\u0263\u0264")
        buf.write("\3\2\2\2\u0264\u0293\3\2\2\2\u0265\u0263\3\2\2\2\u0266")
        buf.write("\u0268\5H%\2\u0267\u0266\3\2\2\2\u0268\u026b\3\2\2\2\u0269")
        buf.write("\u0267\3\2\2\2\u0269\u026a\3\2\2\2\u026a\u0293\3\2\2\2")
        buf.write("\u026b\u0269\3\2\2\2\u026c\u026e\5J&\2\u026d\u026c\3\2")
        buf.write("\2\2\u026e\u0271\3\2\2\2\u026f\u026d\3\2\2\2\u026f\u0270")
        buf.write("\3\2\2\2\u0270\u0293\3\2\2\2\u0271\u026f\3\2\2\2\u0272")
        buf.write("\u0274\5L\'\2\u0273\u0272\3\2\2\2\u0274\u0277\3\2\2\2")
        buf.write("\u0275\u0273\3\2\2\2\u0275\u0276\3\2\2\2\u0276\u0293\3")
        buf.write("\2\2\2\u0277\u0275\3\2\2\2\u0278\u0279\7\f\2\2\u0279\u027a")
        buf.write("\7\u008d\2\2\u027a\u0293\7\u0087\2\2\u027b\u027d\7P\2")
        buf.write("\2\u027c\u027e\7\u008d\2\2\u027d\u027c\3\2\2\2\u027d\u027e")
        buf.write("\3\2\2\2\u027e\u027f\3\2\2\2\u027f\u0293\7\u0085\2\2\u0280")
        buf.write("\u0281\7=\2\2\u0281\u0283\7\u0085\2\2\u0282\u0284\7\u0085")
        buf.write("\2\2\u0283\u0282\3\2\2\2\u0283\u0284\3\2\2\2\u0284\u0293")
        buf.write("\3\2\2\2\u0285\u0286\7\27\2\2\u0286\u0293\7@\2\2\u0287")
        buf.write("\u0288\7\31\2\2\u0288\u0289\7\33\2\2\u0289\u028e\7\u0085")
        buf.write("\2\2\u028a\u028f\7[\2\2\u028b\u028c\7\f\2\2\u028c\u028d")
        buf.write("\7\u008d\2\2\u028d\u028f\7\u0087\2\2\u028e\u028a\3\2\2")
        buf.write("\2\u028e\u028b\3\2\2\2\u028f\u0290\3\2\2\2\u0290\u0293")
        buf.write("\7\u0087\2\2\u0291\u0293\7\34\2\2\u0292\u0245\3\2\2\2")
        buf.write("\u0292\u024b\3\2\2\2\u0292\u0251\3\2\2\2\u0292\u0257\3")
        buf.write("\2\2\2\u0292\u025d\3\2\2\2\u0292\u0263\3\2\2\2\u0292\u0269")
        buf.write("\3\2\2\2\u0292\u026f\3\2\2\2\u0292\u0275\3\2\2\2\u0292")
        buf.write("\u0278\3\2\2\2\u0292\u027b\3\2\2\2\u0292\u0280\3\2\2\2")
        buf.write("\u0292\u0285\3\2\2\2\u0292\u0287\3\2\2\2\u0292\u0291\3")
        buf.write("\2\2\2\u0293;\3\2\2\2\u0294\u0295\7H\2\2\u0295\u0297\5")
        buf.write("^\60\2\u0296\u0298\5^\60\2\u0297\u0296\3\2\2\2\u0297\u0298")
        buf.write("\3\2\2\2\u0298\u0299\3\2\2\2\u0299\u029b\7\u0085\2\2\u029a")
        buf.write("\u029c\7\u0085\2\2\u029b\u029a\3\2\2\2\u029b\u029c\3\2")
        buf.write("\2\2\u029c=\3\2\2\2\u029d\u029e\7E\2\2\u029e\u029f\5^")
        buf.write("\60\2\u029f\u02a0\7\u0085\2\2\u02a0?\3\2\2\2\u02a1\u02a2")
        buf.write("\7I\2\2\u02a2\u02a3\5^\60\2\u02a3\u02a4\5^\60\2\u02a4")
        buf.write("\u02a5\5^\60\2\u02a5\u02a7\7\u0087\2\2\u02a6\u02a8\7\u0084")
        buf.write("\2\2\u02a7\u02a6\3\2\2\2\u02a7\u02a8\3\2\2\2\u02a8\u02a9")
        buf.write("\3\2\2\2\u02a9\u02aa\7\u0087\2\2\u02aaA\3\2\2\2\u02ab")
        buf.write("\u02ac\7J\2\2\u02ac\u02ad\5^\60\2\u02adC\3\2\2\2\u02ae")
        buf.write("\u02af\7K\2\2\u02af\u02b0\5^\60\2\u02b0E\3\2\2\2\u02b1")
        buf.write("\u02b2\7L\2\2\u02b2\u02b3\5^\60\2\u02b3G\3\2\2\2\u02b4")
        buf.write("\u02b5\7M\2\2\u02b5\u02b6\5^\60\2\u02b6I\3\2\2\2\u02b7")
        buf.write("\u02b8\7N\2\2\u02b8\u02b9\7\u0087\2\2\u02b9\u02ba\5^\60")
        buf.write("\2\u02ba\u02bb\5^\60\2\u02bb\u02bc\5^\60\2\u02bc\u02bd")
        buf.write("\5^\60\2\u02bd\u02bf\5^\60\2\u02be\u02c0\5^\60\2\u02bf")
        buf.write("\u02be\3\2\2\2\u02bf\u02c0\3\2\2\2\u02c0K\3\2\2\2\u02c1")
        buf.write("\u02c2\7O\2\2\u02c2\u02c3\5^\60\2\u02c3M\3\2\2\2\u02c4")
        buf.write("\u02c6\5P)\2\u02c5\u02c4\3\2\2\2\u02c6\u02c9\3\2\2\2\u02c7")
        buf.write("\u02c5\3\2\2\2\u02c7\u02c8\3\2\2\2\u02c8\u0311\3\2\2\2")
        buf.write("\u02c9\u02c7\3\2\2\2\u02ca\u02cb\7\f\2\2\u02cb\u02cc\7")
        buf.write("\u008d\2\2\u02cc\u0311\7\u0087\2\2\u02cd\u0311\7R\2\2")
        buf.write("\u02ce\u02cf\7B\2\2\u02cf\u02d1\7\u0083\2\2\u02d0\u02d2")
        buf.write("\7\u0083\2\2\u02d1\u02d0\3\2\2\2\u02d1\u02d2\3\2\2\2\u02d2")
        buf.write("\u02d4\3\2\2\2\u02d3\u02d5\7\u0083\2\2\u02d4\u02d3\3\2")
        buf.write("\2\2\u02d4\u02d5\3\2\2\2\u02d5\u02d7\3\2\2\2\u02d6\u02d8")
        buf.write("\7\u0083\2\2\u02d7\u02d6\3\2\2\2\u02d7\u02d8\3\2\2\2\u02d8")
        buf.write("\u02d9\3\2\2\2\u02d9\u0311\7\u0085\2\2\u02da\u02db\7P")
        buf.write("\2\2\u02db\u02dc\7\u008d\2\2\u02dc\u0311\7\u0085\2\2\u02dd")
        buf.write("\u02de\7=\2\2\u02de\u02df\7\u008d\2\2\u02df\u0311\7\u0085")
        buf.write("\2\2\u02e0\u02e1\7\24\2\2\u02e1\u02e2\7\u008d\2\2\u02e2")
        buf.write("\u0311\7\u0083\2\2\u02e3\u02e4\7S\2\2\u02e4\u02e5\7\u0083")
        buf.write("\2\2\u02e5\u02e6\7\u0083\2\2\u02e6\u02ea\7\u0083\2\2\u02e7")
        buf.write("\u02e8\7\u0083\2\2\u02e8\u02e9\7\u0083\2\2\u02e9\u02eb")
        buf.write("\7\u0083\2\2\u02ea\u02e7\3\2\2\2\u02ea\u02eb\3\2\2\2\u02eb")
        buf.write("\u02ef\3\2\2\2\u02ec\u02ed\7\u0083\2\2\u02ed\u02ee\7\u0083")
        buf.write("\2\2\u02ee\u02f0\7\u0083\2\2\u02ef\u02ec\3\2\2\2\u02ef")
        buf.write("\u02f0\3\2\2\2\u02f0\u02f4\3\2\2\2\u02f1\u02f2\7\u0083")
        buf.write("\2\2\u02f2\u02f3\7\u0083\2\2\u02f3\u02f5\7\u0083\2\2\u02f4")
        buf.write("\u02f1\3\2\2\2\u02f4\u02f5\3\2\2\2\u02f5\u0311\3\2\2\2")
        buf.write("\u02f6\u02f7\7\27\2\2\u02f7\u02f8\7\u008d\2\2\u02f8\u0311")
        buf.write("\7>\2\2\u02f9\u02fa\7\31\2\2\u02fa\u02fb\7\33\2\2\u02fb")
        buf.write("\u0300\7\u0085\2\2\u02fc\u0301\7[\2\2\u02fd\u02fe\7\f")
        buf.write("\2\2\u02fe\u02ff\7\u008d\2\2\u02ff\u0301\7\u0087\2\2\u0300")
        buf.write("\u02fc\3\2\2\2\u0300\u02fd\3\2\2\2\u0301\u0311\3\2\2\2")
        buf.write("\u0302\u0311\7\34\2\2\u0303\u0304\7T\2\2\u0304\u0305\7")
        buf.write("U\2\2\u0305\u0307\7\u0083\2\2\u0306\u0308\7\u0083\2\2")
        buf.write("\u0307\u0306\3\2\2\2\u0307\u0308\3\2\2\2\u0308\u030a\3")
        buf.write("\2\2\2\u0309\u030b\7\u0083\2\2\u030a\u0309\3\2\2\2\u030a")
        buf.write("\u030b\3\2\2\2\u030b\u030d\3\2\2\2\u030c\u030e\7\u0083")
        buf.write("\2\2\u030d\u030c\3\2\2\2\u030d\u030e\3\2\2\2\u030e\u0311")
        buf.write("\3\2\2\2\u030f\u0311\7F\2\2\u0310\u02c7\3\2\2\2\u0310")
        buf.write("\u02ca\3\2\2\2\u0310\u02cd\3\2\2\2\u0310\u02ce\3\2\2\2")
        buf.write("\u0310\u02da\3\2\2\2\u0310\u02dd\3\2\2\2\u0310\u02e0\3")
        buf.write("\2\2\2\u0310\u02e3\3\2\2\2\u0310\u02f6\3\2\2\2\u0310\u02f9")
        buf.write("\3\2\2\2\u0310\u0302\3\2\2\2\u0310\u0303\3\2\2\2\u0310")
        buf.write("\u030f\3\2\2\2\u0311O\3\2\2\2\u0312\u0313\7\7\2\2\u0313")
        buf.write("\u0314\5^\60\2\u0314\u0315\5^\60\2\u0315\u0316\5^\60\2")
        buf.write("\u0316\u031c\5^\60\2\u0317\u0318\5^\60\2\u0318\u0319\5")
        buf.write("^\60\2\u0319\u031a\5^\60\2\u031a\u031b\5^\60\2\u031b\u031d")
        buf.write("\3\2\2\2\u031c\u0317\3\2\2\2\u031c\u031d\3\2\2\2\u031d")
        buf.write("\u0323\3\2\2\2\u031e\u031f\5^\60\2\u031f\u0320\5^\60\2")
        buf.write("\u0320\u0321\5^\60\2\u0321\u0322\5^\60\2\u0322\u0324\3")
        buf.write("\2\2\2\u0323\u031e\3\2\2\2\u0323\u0324\3\2\2\2\u0324\u032a")
        buf.write("\3\2\2\2\u0325\u0326\5^\60\2\u0326\u0327\5^\60\2\u0327")
        buf.write("\u0328\5^\60\2\u0328\u0329\5^\60\2\u0329\u032b\3\2\2\2")
        buf.write("\u032a\u0325\3\2\2\2\u032a\u032b\3\2\2\2\u032bQ\3\2\2")
        buf.write("\2\u032c\u032e\5T+\2\u032d\u032c\3\2\2\2\u032e\u0331\3")
        buf.write("\2\2\2\u032f\u032d\3\2\2\2\u032f\u0330\3\2\2\2\u0330\u0349")
        buf.write("\3\2\2\2\u0331\u032f\3\2\2\2\u0332\u0333\7\f\2\2\u0333")
        buf.write("\u0334\7\u008d\2\2\u0334\u0349\7\u0087\2\2\u0335\u0336")
        buf.write("\7<\2\2\u0336\u0337\7\u0085\2\2\u0337\u0338\7\u0085\2")
        buf.write("\2\u0338\u0339\7\u0085\2\2\u0339\u033a\7\u0085\2\2\u033a")
        buf.write("\u0349\7\u0085\2\2\u033b\u033c\7=\2\2\u033c\u033d\7\u008d")
        buf.write("\2\2\u033d\u0349\7\u0085\2\2\u033e\u033f\7\24\2\2\u033f")
        buf.write("\u0340\7\u008d\2\2\u0340\u0349\7\u0083\2\2\u0341\u0342")
        buf.write("\7\27\2\2\u0342\u0343\7\u008d\2\2\u0343\u0349\7>\2\2\u0344")
        buf.write("\u0345\7\31\2\2\u0345\u0346\7\33\2\2\u0346\u0349\7\u0085")
        buf.write("\2\2\u0347\u0349\7\34\2\2\u0348\u032f\3\2\2\2\u0348\u0332")
        buf.write("\3\2\2\2\u0348\u0335\3\2\2\2\u0348\u033b\3\2\2\2\u0348")
        buf.write("\u033e\3\2\2\2\u0348\u0341\3\2\2\2\u0348\u0344\3\2\2\2")
        buf.write("\u0348\u0347\3\2\2\2\u0349S\3\2\2\2\u034a\u034b\7\7\2")
        buf.write("\2\u034b\u034c\5^\60\2\u034c\u034d\5^\60\2\u034d\u034e")
        buf.write("\5^\60\2\u034e\u034f\5^\60\2\u034f\u0350\5^\60\2\u0350")
        buf.write("\u0351\5^\60\2\u0351\u0352\7\u0085\2\2\u0352\u0353\7\u0085")
        buf.write("\2\2\u0353U\3\2\2\2\u0354\u0356\5X-\2\u0355\u0354\3\2")
        buf.write("\2\2\u0356\u0359\3\2\2\2\u0357\u0355\3\2\2\2\u0357\u0358")
        buf.write("\3\2\2\2\u0358\u0373\3\2\2\2\u0359\u0357\3\2\2\2\u035a")
        buf.write("\u035b\7\f\2\2\u035b\u035c\7\u008d\2\2\u035c\u0373\7\u0087")
        buf.write("\2\2\u035d\u035e\7<\2\2\u035e\u035f\7\u0085\2\2\u035f")
        buf.write("\u0360\7\u0085\2\2\u0360\u0361\7\u0085\2\2\u0361\u0362")
        buf.write("\7\u0085\2\2\u0362\u0363\7\u0085\2\2\u0363\u0364\7\u0085")
        buf.write("\2\2\u0364\u0373\7\u0085\2\2\u0365\u0366\7=\2\2\u0366")
        buf.write("\u0367\7\u008d\2\2\u0367\u0373\7\u0085\2\2\u0368\u0369")
        buf.write("\7\24\2\2\u0369\u036a\7\u008d\2\2\u036a\u0373\7\u0083")
        buf.write("\2\2\u036b\u036c\7\27\2\2\u036c\u036d\7\u008d\2\2\u036d")
        buf.write("\u0373\7>\2\2\u036e\u036f\7\31\2\2\u036f\u0370\7\33\2")
        buf.write("\2\u0370\u0373\7\u0085\2\2\u0371\u0373\7\34\2\2\u0372")
        buf.write("\u0357\3\2\2\2\u0372\u035a\3\2\2\2\u0372\u035d\3\2\2\2")
        buf.write("\u0372\u0365\3\2\2\2\u0372\u0368\3\2\2\2\u0372\u036b\3")
        buf.write("\2\2\2\u0372\u036e\3\2\2\2\u0372\u0371\3\2\2\2\u0373W")
        buf.write("\3\2\2\2\u0374\u0375\7\7\2\2\u0375\u0376\5^\60\2\u0376")
        buf.write("\u0377\5^\60\2\u0377\u0378\5^\60\2\u0378\u0379\5^\60\2")
        buf.write("\u0379\u037a\5^\60\2\u037a\u037b\5^\60\2\u037b\u037c\5")
        buf.write("^\60\2\u037c\u037d\5^\60\2\u037d\u037e\7\u0085\2\2\u037e")
        buf.write("\u037f\7\u0085\2\2\u037fY\3\2\2\2\u0380\u0382\5\\/\2\u0381")
        buf.write("\u0380\3\2\2\2\u0382\u0385\3\2\2\2\u0383\u0381\3\2\2\2")
        buf.write("\u0383\u0384\3\2\2\2\u0384\u03ac\3\2\2\2\u0385\u0383\3")
        buf.write("\2\2\2\u0386\u0387\7\f\2\2\u0387\u0388\7\u008d\2\2\u0388")
        buf.write("\u03ac\7\u0087\2\2\u0389\u038a\7Y\2\2\u038a\u038b\7\u008d")
        buf.write("\2\2\u038b\u03ac\7\u0087\2\2\u038c\u038d\7B\2\2\u038d")
        buf.write("\u038e\7\u0083\2\2\u038e\u038f\7\u0083\2\2\u038f\u03ac")
        buf.write("\7\u0085\2\2\u0390\u0391\7P\2\2\u0391\u0392\7\u008d\2")
        buf.write("\2\u0392\u03ac\7\u0085\2\2\u0393\u0394\7=\2\2\u0394\u0395")
        buf.write("\7\u008d\2\2\u0395\u03ac\7\u0085\2\2\u0396\u0397\7\24")
        buf.write("\2\2\u0397\u0398\7\u008d\2\2\u0398\u03ac\7\u0083\2\2\u0399")
        buf.write("\u039a\7\27\2\2\u039a\u039b\7\u008d\2\2\u039b\u03ac\7")
        buf.write(">\2\2\u039c\u039d\7\31\2\2\u039d\u039e\7\33\2\2\u039e")
        buf.write("\u03a3\7\u0085\2\2\u039f\u03a4\7[\2\2\u03a0\u03a1\7\f")
        buf.write("\2\2\u03a1\u03a2\7\u008d\2\2\u03a2\u03a4\7\u0087\2\2\u03a3")
        buf.write("\u039f\3\2\2\2\u03a3\u03a0\3\2\2\2\u03a4\u03ac\3\2\2\2")
        buf.write("\u03a5\u03ac\7\34\2\2\u03a6\u03a7\7T\2\2\u03a7\u03a8\7")
        buf.write("Z\2\2\u03a8\u03a9\7\u0083\2\2\u03a9\u03ac\7\u0083\2\2")
        buf.write("\u03aa\u03ac\7F\2\2\u03ab\u0383\3\2\2\2\u03ab\u0386\3")
        buf.write("\2\2\2\u03ab\u0389\3\2\2\2\u03ab\u038c\3\2\2\2\u03ab\u0390")
        buf.write("\3\2\2\2\u03ab\u0393\3\2\2\2\u03ab\u0396\3\2\2\2\u03ab")
        buf.write("\u0399\3\2\2\2\u03ab\u039c\3\2\2\2\u03ab\u03a5\3\2\2\2")
        buf.write("\u03ab\u03a6\3\2\2\2\u03ab\u03aa\3\2\2\2\u03ac[\3\2\2")
        buf.write("\2\u03ad\u03ae\7\7\2\2\u03ae\u03af\5^\60\2\u03af\u03b0")
        buf.write("\5^\60\2\u03b0\u03b1\5^\60\2\u03b1\u03b2\5^\60\2\u03b2")
        buf.write("\u03b3\5^\60\2\u03b3\u03b4\5^\60\2\u03b4\u03b5\5^\60\2")
        buf.write("\u03b5\u03b6\5^\60\2\u03b6\u03b7\5^\60\2\u03b7\u03b8\5")
        buf.write("^\60\2\u03b8\u03ba\5^\60\2\u03b9\u03bb\5^\60\2\u03ba\u03b9")
        buf.write("\3\2\2\2\u03ba\u03bb\3\2\2\2\u03bb]\3\2\2\2\u03bc\u03bd")
        buf.write("\7\u008a\2\2\u03bd\u03be\5`\61\2\u03be\u03bf\7\u008b\2")
        buf.write("\2\u03bf_\3\2\2\2\u03c0\u03c5\5b\62\2\u03c1\u03c2\7~\2")
        buf.write("\2\u03c2\u03c4\5b\62\2\u03c3\u03c1\3\2\2\2\u03c4\u03c7")
        buf.write("\3\2\2\2\u03c5\u03c3\3\2\2\2\u03c5\u03c6\3\2\2\2\u03c6")
        buf.write("a\3\2\2\2\u03c7\u03c5\3\2\2\2\u03c8\u03cd\5d\63\2\u03c9")
        buf.write("\u03ca\7\177\2\2\u03ca\u03cc\5d\63\2\u03cb\u03c9\3\2\2")
        buf.write("\2\u03cc\u03cf\3\2\2\2\u03cd\u03cb\3\2\2\2\u03cd\u03ce")
        buf.write("\3\2\2\2\u03cec\3\2\2\2\u03cf\u03cd\3\2\2\2\u03d0\u03d1")
        buf.write("\b\63\1\2\u03d1\u03d2\7\u008a\2\2\u03d2\u03d3\5`\61\2")
        buf.write("\u03d3\u03d4\7\u008b\2\2\u03d4\u045a\3\2\2\2\u03d5\u045a")
        buf.write("\7[\2\2\u03d6\u03d7\7]\2\2\u03d7\u03d8\t\3\2\2\u03d8\u03d9")
        buf.write("\t\4\2\2\u03d9\u045a\t\3\2\2\u03da\u03dc\7^\2\2\u03db")
        buf.write("\u03dd\7\u0097\2\2\u03dc\u03db\3\2\2\2\u03dc\u03dd\3\2")
        buf.write("\2\2\u03dd\u03de\3\2\2\2\u03de\u03df\7\u0098\2\2\u03df")
        buf.write("\u03e0\7\u0099\2\2\u03e0\u045a\7\u0085\2\2\u03e1\u03e2")
        buf.write("\7_\2\2\u03e2\u045a\5d\63 \u03e3\u03e4\7`\2\2\u03e4\u045a")
        buf.write("\5d\63\37\u03e5\u03e6\7a\2\2\u03e6\u045a\5d\63\36\u03e7")
        buf.write("\u03ee\7b\2\2\u03e8\u03ef\7\u0088\2\2\u03e9\u03ec\7\u0087")
        buf.write("\2\2\u03ea\u03eb\7\u008c\2\2\u03eb\u03ed\7\u0087\2\2\u03ec")
        buf.write("\u03ea\3\2\2\2\u03ec\u03ed\3\2\2\2\u03ed\u03ef\3\2\2\2")
        buf.write("\u03ee\u03e8\3\2\2\2\u03ee\u03e9\3\2\2\2\u03ef\u045a\3")
        buf.write("\2\2\2\u03f0\u03f1\7c\2\2\u03f1\u03f2\7\u0085\2\2\u03f2")
        buf.write("\u03f3\7\u0085\2\2\u03f3\u03f4\7\u0085\2\2\u03f4\u03f5")
        buf.write("\7\u0085\2\2\u03f5\u03f6\7\u0085\2\2\u03f6\u045a\7\u0085")
        buf.write("\2\2\u03f7\u045a\7d\2\2\u03f8\u03f9\7e\2\2\u03f9\u045a")
        buf.write("\7\u0083\2\2\u03fa\u045a\7f\2\2\u03fb\u0402\7g\2\2\u03fc")
        buf.write("\u0403\7\u0088\2\2\u03fd\u0400\7\u0087\2\2\u03fe\u03ff")
        buf.write("\7\u008c\2\2\u03ff\u0401\7\u0087\2\2\u0400\u03fe\3\2\2")
        buf.write("\2\u0400\u0401\3\2\2\2\u0401\u0403\3\2\2\2\u0402\u03fc")
        buf.write("\3\2\2\2\u0402\u03fd\3\2\2\2\u0403\u045a\3\2\2\2\u0404")
        buf.write("\u045a\7h\2\2\u0405\u0406\7\u0080\2\2\u0406\u045a\5d\63")
        buf.write("\26\u0407\u0408\7i\2\2\u0408\u0409\7\u008a\2\2\u0409\u040b")
        buf.write("\7\u0085\2\2\u040a\u040c\7\u0081\2\2\u040b\u040a\3\2\2")
        buf.write("\2\u040b\u040c\3\2\2\2\u040c\u040d\3\2\2\2\u040d\u040f")
        buf.write("\7\u0085\2\2\u040e\u0410\7\u0081\2\2\u040f\u040e\3\2\2")
        buf.write("\2\u040f\u0410\3\2\2\2\u0410\u0411\3\2\2\2\u0411\u0412")
        buf.write("\7\u0085\2\2\u0412\u0413\7\u008b\2\2\u0413\u0414\7j\2")
        buf.write("\2\u0414\u045a\7\u0085\2\2\u0415\u0416\7i\2\2\u0416\u0417")
        buf.write("\7\u008a\2\2\u0417\u0418\7|\2\2\u0418\u0419\7\u008d\2")
        buf.write("\2\u0419\u041b\5^\60\2\u041a\u041c\7\u0081\2\2\u041b\u041a")
        buf.write("\3\2\2\2\u041b\u041c\3\2\2\2\u041c\u0420\3\2\2\2\u041d")
        buf.write("\u041e\7}\2\2\u041e\u041f\7\u008d\2\2\u041f\u0421\5^\60")
        buf.write("\2\u0420\u041d\3\2\2\2\u0420\u0421\3\2\2\2\u0421\u0422")
        buf.write("\3\2\2\2\u0422\u0423\7\u008b\2\2\u0423\u0424\7j\2\2\u0424")
        buf.write("\u0425\7\u0085\2\2\u0425\u045a\3\2\2\2\u0426\u045a\7k")
        buf.write("\2\2\u0427\u045a\7l\2\2\u0428\u042f\7m\2\2\u0429\u0430")
        buf.write("\7\u0089\2\2\u042a\u042d\7\u0083\2\2\u042b\u042c\7\u008c")
        buf.write("\2\2\u042c\u042e\7\u0083\2\2\u042d\u042b\3\2\2\2\u042d")
        buf.write("\u042e\3\2\2\2\u042e\u0430\3\2\2\2\u042f\u0429\3\2\2\2")
        buf.write("\u042f\u042a\3\2\2\2\u0430\u045a\3\2\2\2\u0431\u0438\7")
        buf.write("n\2\2\u0432\u0439\7\u0088\2\2\u0433\u0436\7\u0087\2\2")
        buf.write("\u0434\u0435\7\u008c\2\2\u0435\u0437\7\u0087\2\2\u0436")
        buf.write("\u0434\3\2\2\2\u0436\u0437\3\2\2\2\u0437\u0439\3\2\2\2")
        buf.write("\u0438\u0432\3\2\2\2\u0438\u0433\3\2\2\2\u0439\u045a\3")
        buf.write("\2\2\2\u043a\u0446\7p\2\2\u043b\u0447\7\u0088\2\2\u043c")
        buf.write("\u043f\7\u0087\2\2\u043d\u043e\7\u008c\2\2\u043e\u0440")
        buf.write("\7\u0087\2\2\u043f\u043d\3\2\2\2\u043f\u0440\3\2\2\2\u0440")
        buf.write("\u0447\3\2\2\2\u0441\u0444\7\u0086\2\2\u0442\u0443\7\u008c")
        buf.write("\2\2\u0443\u0445\7\u0086\2\2\u0444\u0442\3\2\2\2\u0444")
        buf.write("\u0445\3\2\2\2\u0445\u0447\3\2\2\2\u0446\u043b\3\2\2\2")
        buf.write("\u0446\u043c\3\2\2\2\u0446\u0441\3\2\2\2\u0447\u045a\3")
        buf.write("\2\2\2\u0448\u0449\7q\2\2\u0449\u044a\7\u0085\2\2\u044a")
        buf.write("\u044b\7\u0085\2\2\u044b\u044c\7\u0085\2\2\u044c\u044d")
        buf.write("\7\u0085\2\2\u044d\u044e\7\u0085\2\2\u044e\u045a\7\u0085")
        buf.write("\2\2\u044f\u045a\7r\2\2\u0450\u045a\7s\2\2\u0451\u045a")
        buf.write("\7t\2\2\u0452\u045a\7u\2\2\u0453\u045a\7v\2\2\u0454\u045a")
        buf.write("\7w\2\2\u0455\u045a\7x\2\2\u0456\u045a\7y\2\2\u0457\u045a")
        buf.write("\7z\2\2\u0458\u045a\7{\2\2\u0459\u03d0\3\2\2\2\u0459\u03d5")
        buf.write("\3\2\2\2\u0459\u03d6\3\2\2\2\u0459\u03da\3\2\2\2\u0459")
        buf.write("\u03e1\3\2\2\2\u0459\u03e3\3\2\2\2\u0459\u03e5\3\2\2\2")
        buf.write("\u0459\u03e7\3\2\2\2\u0459\u03f0\3\2\2\2\u0459\u03f7\3")
        buf.write("\2\2\2\u0459\u03f8\3\2\2\2\u0459\u03fa\3\2\2\2\u0459\u03fb")
        buf.write("\3\2\2\2\u0459\u0404\3\2\2\2\u0459\u0405\3\2\2\2\u0459")
        buf.write("\u0407\3\2\2\2\u0459\u0415\3\2\2\2\u0459\u0426\3\2\2\2")
        buf.write("\u0459\u0427\3\2\2\2\u0459\u0428\3\2\2\2\u0459\u0431\3")
        buf.write("\2\2\2\u0459\u043a\3\2\2\2\u0459\u0448\3\2\2\2\u0459\u044f")
        buf.write("\3\2\2\2\u0459\u0450\3\2\2\2\u0459\u0451\3\2\2\2\u0459")
        buf.write("\u0452\3\2\2\2\u0459\u0453\3\2\2\2\u0459\u0454\3\2\2\2")
        buf.write("\u0459\u0455\3\2\2\2\u0459\u0456\3\2\2\2\u0459\u0457\3")
        buf.write("\2\2\2\u0459\u0458\3\2\2\2\u045a\u0463\3\2\2\2\u045b\u045c")
        buf.write("\f#\2\2\u045c\u045d\7\\\2\2\u045d\u0462\7\u0085\2\2\u045e")
        buf.write("\u045f\f\17\2\2\u045f\u0460\7o\2\2\u0460\u0462\7\u0085")
        buf.write("\2\2\u0461\u045b\3\2\2\2\u0461\u045e\3\2\2\2\u0462\u0465")
        buf.write("\3\2\2\2\u0463\u0461\3\2\2\2\u0463\u0464\3\2\2\2\u0464")
        buf.write("e\3\2\2\2\u0465\u0463\3\2\2\2crtz\u0080\u0086\u008c\u0092")
        buf.write("\u00d0\u00f8\u0118\u0133\u013a\u0148\u0155\u015a\u016b")
        buf.write("\u017b\u0183\u018d\u0191\u0198\u019d\u019f\u01a2\u01a7")
        buf.write("\u01be\u01ca\u01cf\u01e0\u01f2\u01f5\u0201\u0207\u020c")
        buf.write("\u022d\u0232\u0245\u024b\u0251\u0257\u025d\u0263\u0269")
        buf.write("\u026f\u0275\u027d\u0283\u028e\u0292\u0297\u029b\u02a7")
        buf.write("\u02bf\u02c7\u02d1\u02d4\u02d7\u02ea\u02ef\u02f4\u0300")
        buf.write("\u0307\u030a\u030d\u0310\u031c\u0323\u032a\u032f\u0348")
        buf.write("\u0357\u0372\u0383\u03a3\u03ab\u03ba\u03c5\u03cd\u03dc")
        buf.write("\u03ec\u03ee\u0400\u0402\u040b\u040f\u041b\u0420\u042d")
        buf.write("\u042f\u0436\u0438\u043f\u0444\u0446\u0459\u0461\u0463")
        return buf.getvalue()


class CnsMRParser ( Parser ):

    grammarFileName = "CnsMRParser.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "'?'", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "','", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "'('", "')'", "':'", "'='", 
                     "'<'", "'>'", "'<='", "'>='", "'#'" ]

    symbolicNames = [ "<INVALID>", "Set", "End", "Noe", "Analysis", "Assign", 
                      "Asymptote", "Averaging", "Bgig", "Ceiling", "Classification", 
                      "CountViol", "Cv", "Den", "Distribute", "Ensemble", 
                      "Monomers", "Ncount", "Nrestraints", "Outd", "Partition", 
                      "Potential", "Predict", "Print", "Raverage", "Threshold", 
                      "Reset", "Rswitch", "Scale", "SoExponent", "SqConstant", 
                      "SqExponent", "SqOffset", "Taverage", "Temperature", 
                      "Noe_analysis", "Initialize", "Update", "Gamma", "Kappa", 
                      "Noe_avr_methods", "Noe_potential", "Cutoff", "Cuton", 
                      "From", "To", "Restraints", "Dihedral", "Nassign", 
                      "Print_any", "Plane", "Group", "Selection", "Weight", 
                      "Harmonic", "Exponent", "Normal", "Sanisotropy", "Coefficients", 
                      "ForceConstant", "Rdc_potential", "Coupling", "Coupling_potential", 
                      "Carbon", "Expectation", "PhiStep", "PsiStep", "Rcoil", 
                      "Zero", "Proton", "Observed", "Anisotropy", "Amides", 
                      "Carbons", "Nitrogens", "Oxygens", "RingAtoms", "AlphasAndAmides", 
                      "Error", "Conformation", "Compressed", "Phase", "Size", 
                      "Dimensions", "Danisotropy", "OneBond", "AngleDb", 
                      "DerivFlag", "Angle_dihedral", "All", "Around", "Atom", 
                      "Attribute", "BondedTo", "ByGroup", "ByRes", "Chemical", 
                      "Fbox", "Hydrogen", "Id", "Known", "Name", "NONE", 
                      "Point", "Cut", "Previous", "Pseudo", "Residue", "Resname", 
                      "Saround", "SegIdentifier", "Sfbox", "Store_1", "Store_2", 
                      "Store_3", "Store_4", "Store_5", "Store_6", "Store_7", 
                      "Store_8", "Store_9", "Tag", "Tail", "Head", "Or_op", 
                      "And_op", "Not_op", "Comma", "Complex", "Integer", 
                      "Logical", "Real", "Double_quote_string", "Simple_name", 
                      "Simple_names", "Integers", "L_paren", "R_paren", 
                      "Colon", "Equ_op", "Lt_op", "Gt_op", "Leq_op", "Geq_op", 
                      "Neq_op", "SPACE", "COMMENT", "LINE_COMMENT", "SET_VARIABLE", 
                      "Abs", "Attr_properties", "Comparison_ops", "SPACE_ATTR" ]

    RULE_cns_mr = 0
    RULE_distance_restraint = 1
    RULE_dihedral_angle_restraint = 2
    RULE_plane_restraint = 3
    RULE_harmonic_restraint = 4
    RULE_rdc_restraint = 5
    RULE_coupling_restraint = 6
    RULE_carbon_shift_restraint = 7
    RULE_proton_shift_restraint = 8
    RULE_conformation_db_restraint = 9
    RULE_diffusion_anisotropy_restraint = 10
    RULE_one_bond_coupling_restraint = 11
    RULE_angle_db_restraint = 12
    RULE_noe_statement = 13
    RULE_noe_assign = 14
    RULE_predict_statement = 15
    RULE_dihedral_statement = 16
    RULE_dihedral_assign = 17
    RULE_plane_statement = 18
    RULE_group_statement = 19
    RULE_harmonic_statement = 20
    RULE_sani_statement = 21
    RULE_sani_assign = 22
    RULE_coupling_statement = 23
    RULE_coup_assign = 24
    RULE_carbon_shift_statement = 25
    RULE_carbon_shift_assign = 26
    RULE_carbon_shift_rcoil = 27
    RULE_proton_shift_statement = 28
    RULE_observed = 29
    RULE_proton_shift_rcoil = 30
    RULE_proton_shift_anisotropy = 31
    RULE_proton_shift_amides = 32
    RULE_proton_shift_carbons = 33
    RULE_proton_shift_nitrogens = 34
    RULE_proton_shift_oxygens = 35
    RULE_proton_shift_ring_atoms = 36
    RULE_proton_shift_alphas_and_amides = 37
    RULE_conformation_statement = 38
    RULE_conf_assign = 39
    RULE_diffusion_statement = 40
    RULE_dani_assign = 41
    RULE_one_bond_coupling_statement = 42
    RULE_one_bond_assign = 43
    RULE_angle_db_statement = 44
    RULE_angle_db_assign = 45
    RULE_selection = 46
    RULE_selection_expression = 47
    RULE_term = 48
    RULE_factor = 49

    ruleNames =  [ "cns_mr", "distance_restraint", "dihedral_angle_restraint", 
                   "plane_restraint", "harmonic_restraint", "rdc_restraint", 
                   "coupling_restraint", "carbon_shift_restraint", "proton_shift_restraint", 
                   "conformation_db_restraint", "diffusion_anisotropy_restraint", 
                   "one_bond_coupling_restraint", "angle_db_restraint", 
                   "noe_statement", "noe_assign", "predict_statement", "dihedral_statement", 
                   "dihedral_assign", "plane_statement", "group_statement", 
                   "harmonic_statement", "sani_statement", "sani_assign", 
                   "coupling_statement", "coup_assign", "carbon_shift_statement", 
                   "carbon_shift_assign", "carbon_shift_rcoil", "proton_shift_statement", 
                   "observed", "proton_shift_rcoil", "proton_shift_anisotropy", 
                   "proton_shift_amides", "proton_shift_carbons", "proton_shift_nitrogens", 
                   "proton_shift_oxygens", "proton_shift_ring_atoms", "proton_shift_alphas_and_amides", 
                   "conformation_statement", "conf_assign", "diffusion_statement", 
                   "dani_assign", "one_bond_coupling_statement", "one_bond_assign", 
                   "angle_db_statement", "angle_db_assign", "selection", 
                   "selection_expression", "term", "factor" ]

    EOF = Token.EOF
    Set=1
    End=2
    Noe=3
    Analysis=4
    Assign=5
    Asymptote=6
    Averaging=7
    Bgig=8
    Ceiling=9
    Classification=10
    CountViol=11
    Cv=12
    Den=13
    Distribute=14
    Ensemble=15
    Monomers=16
    Ncount=17
    Nrestraints=18
    Outd=19
    Partition=20
    Potential=21
    Predict=22
    Print=23
    Raverage=24
    Threshold=25
    Reset=26
    Rswitch=27
    Scale=28
    SoExponent=29
    SqConstant=30
    SqExponent=31
    SqOffset=32
    Taverage=33
    Temperature=34
    Noe_analysis=35
    Initialize=36
    Update=37
    Gamma=38
    Kappa=39
    Noe_avr_methods=40
    Noe_potential=41
    Cutoff=42
    Cuton=43
    From=44
    To=45
    Restraints=46
    Dihedral=47
    Nassign=48
    Print_any=49
    Plane=50
    Group=51
    Selection=52
    Weight=53
    Harmonic=54
    Exponent=55
    Normal=56
    Sanisotropy=57
    Coefficients=58
    ForceConstant=59
    Rdc_potential=60
    Coupling=61
    Coupling_potential=62
    Carbon=63
    Expectation=64
    PhiStep=65
    PsiStep=66
    Rcoil=67
    Zero=68
    Proton=69
    Observed=70
    Anisotropy=71
    Amides=72
    Carbons=73
    Nitrogens=74
    Oxygens=75
    RingAtoms=76
    AlphasAndAmides=77
    Error=78
    Conformation=79
    Compressed=80
    Phase=81
    Size=82
    Dimensions=83
    Danisotropy=84
    OneBond=85
    AngleDb=86
    DerivFlag=87
    Angle_dihedral=88
    All=89
    Around=90
    Atom=91
    Attribute=92
    BondedTo=93
    ByGroup=94
    ByRes=95
    Chemical=96
    Fbox=97
    Hydrogen=98
    Id=99
    Known=100
    Name=101
    NONE=102
    Point=103
    Cut=104
    Previous=105
    Pseudo=106
    Residue=107
    Resname=108
    Saround=109
    SegIdentifier=110
    Sfbox=111
    Store_1=112
    Store_2=113
    Store_3=114
    Store_4=115
    Store_5=116
    Store_6=117
    Store_7=118
    Store_8=119
    Store_9=120
    Tag=121
    Tail=122
    Head=123
    Or_op=124
    And_op=125
    Not_op=126
    Comma=127
    Complex=128
    Integer=129
    Logical=130
    Real=131
    Double_quote_string=132
    Simple_name=133
    Simple_names=134
    Integers=135
    L_paren=136
    R_paren=137
    Colon=138
    Equ_op=139
    Lt_op=140
    Gt_op=141
    Leq_op=142
    Geq_op=143
    Neq_op=144
    SPACE=145
    COMMENT=146
    LINE_COMMENT=147
    SET_VARIABLE=148
    Abs=149
    Attr_properties=150
    Comparison_ops=151
    SPACE_ATTR=152

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.9")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class Cns_mrContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EOF(self):
            return self.getToken(CnsMRParser.EOF, 0)

        def distance_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Distance_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Distance_restraintContext,i)


        def dihedral_angle_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Dihedral_angle_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Dihedral_angle_restraintContext,i)


        def plane_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Plane_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Plane_restraintContext,i)


        def harmonic_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Harmonic_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Harmonic_restraintContext,i)


        def rdc_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Rdc_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Rdc_restraintContext,i)


        def coupling_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Coupling_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Coupling_restraintContext,i)


        def carbon_shift_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Carbon_shift_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Carbon_shift_restraintContext,i)


        def proton_shift_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Proton_shift_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Proton_shift_restraintContext,i)


        def conformation_db_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Conformation_db_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Conformation_db_restraintContext,i)


        def diffusion_anisotropy_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Diffusion_anisotropy_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Diffusion_anisotropy_restraintContext,i)


        def one_bond_coupling_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.One_bond_coupling_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.One_bond_coupling_restraintContext,i)


        def angle_db_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Angle_db_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Angle_db_restraintContext,i)


        def noe_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Noe_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Noe_assignContext,i)


        def dihedral_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Dihedral_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Dihedral_assignContext,i)


        def sani_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Sani_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Sani_assignContext,i)


        def plane_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Plane_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Plane_statementContext,i)


        def coup_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Coup_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Coup_assignContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_cns_mr

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCns_mr" ):
                listener.enterCns_mr(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCns_mr" ):
                listener.exitCns_mr(self)




    def cns_mr(self):

        localctx = CnsMRParser.Cns_mrContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_cns_mr)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 114
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << CnsMRParser.Noe) | (1 << CnsMRParser.Restraints) | (1 << CnsMRParser.Sanisotropy) | (1 << CnsMRParser.Coupling) | (1 << CnsMRParser.Carbon))) != 0) or ((((_la - 69)) & ~0x3f) == 0 and ((1 << (_la - 69)) & ((1 << (CnsMRParser.Proton - 69)) | (1 << (CnsMRParser.Conformation - 69)) | (1 << (CnsMRParser.Danisotropy - 69)) | (1 << (CnsMRParser.OneBond - 69)) | (1 << (CnsMRParser.AngleDb - 69)))) != 0):
                self.state = 112
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,0,self._ctx)
                if la_ == 1:
                    self.state = 100
                    self.distance_restraint()
                    pass

                elif la_ == 2:
                    self.state = 101
                    self.dihedral_angle_restraint()
                    pass

                elif la_ == 3:
                    self.state = 102
                    self.plane_restraint()
                    pass

                elif la_ == 4:
                    self.state = 103
                    self.harmonic_restraint()
                    pass

                elif la_ == 5:
                    self.state = 104
                    self.rdc_restraint()
                    pass

                elif la_ == 6:
                    self.state = 105
                    self.coupling_restraint()
                    pass

                elif la_ == 7:
                    self.state = 106
                    self.carbon_shift_restraint()
                    pass

                elif la_ == 8:
                    self.state = 107
                    self.proton_shift_restraint()
                    pass

                elif la_ == 9:
                    self.state = 108
                    self.conformation_db_restraint()
                    pass

                elif la_ == 10:
                    self.state = 109
                    self.diffusion_anisotropy_restraint()
                    pass

                elif la_ == 11:
                    self.state = 110
                    self.one_bond_coupling_restraint()
                    pass

                elif la_ == 12:
                    self.state = 111
                    self.angle_db_restraint()
                    pass


                self.state = 116
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 120
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,2,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 117
                    self.noe_assign() 
                self.state = 122
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,2,self._ctx)

            self.state = 126
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,3,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 123
                    self.dihedral_assign() 
                self.state = 128
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,3,self._ctx)

            self.state = 132
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,4,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 129
                    self.sani_assign() 
                self.state = 134
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,4,self._ctx)

            self.state = 138
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << CnsMRParser.Initialize) | (1 << CnsMRParser.Print_any) | (1 << CnsMRParser.Group))) != 0):
                self.state = 135
                self.plane_statement()
                self.state = 140
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 144
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==CnsMRParser.Assign:
                self.state = 141
                self.coup_assign()
                self.state = 146
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 147
            self.match(CnsMRParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Distance_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Noe(self):
            return self.getToken(CnsMRParser.Noe, 0)

        def noe_statement(self):
            return self.getTypedRuleContext(CnsMRParser.Noe_statementContext,0)


        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_distance_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDistance_restraint" ):
                listener.enterDistance_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDistance_restraint" ):
                listener.exitDistance_restraint(self)




    def distance_restraint(self):

        localctx = CnsMRParser.Distance_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_distance_restraint)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 149
            self.match(CnsMRParser.Noe)
            self.state = 150
            self.noe_statement()
            self.state = 151
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dihedral_angle_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Restraints(self):
            return self.getToken(CnsMRParser.Restraints, 0)

        def Dihedral(self):
            return self.getToken(CnsMRParser.Dihedral, 0)

        def dihedral_statement(self):
            return self.getTypedRuleContext(CnsMRParser.Dihedral_statementContext,0)


        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_dihedral_angle_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDihedral_angle_restraint" ):
                listener.enterDihedral_angle_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDihedral_angle_restraint" ):
                listener.exitDihedral_angle_restraint(self)




    def dihedral_angle_restraint(self):

        localctx = CnsMRParser.Dihedral_angle_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_dihedral_angle_restraint)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 153
            self.match(CnsMRParser.Restraints)
            self.state = 154
            self.match(CnsMRParser.Dihedral)
            self.state = 155
            self.dihedral_statement()
            self.state = 156
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Plane_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Restraints(self):
            return self.getToken(CnsMRParser.Restraints, 0)

        def Plane(self):
            return self.getToken(CnsMRParser.Plane, 0)

        def plane_statement(self):
            return self.getTypedRuleContext(CnsMRParser.Plane_statementContext,0)


        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_plane_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPlane_restraint" ):
                listener.enterPlane_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPlane_restraint" ):
                listener.exitPlane_restraint(self)




    def plane_restraint(self):

        localctx = CnsMRParser.Plane_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_plane_restraint)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 158
            self.match(CnsMRParser.Restraints)
            self.state = 159
            self.match(CnsMRParser.Plane)
            self.state = 160
            self.plane_statement()
            self.state = 161
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Harmonic_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Restraints(self):
            return self.getToken(CnsMRParser.Restraints, 0)

        def Harmonic(self):
            return self.getToken(CnsMRParser.Harmonic, 0)

        def harmonic_statement(self):
            return self.getTypedRuleContext(CnsMRParser.Harmonic_statementContext,0)


        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_harmonic_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterHarmonic_restraint" ):
                listener.enterHarmonic_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitHarmonic_restraint" ):
                listener.exitHarmonic_restraint(self)




    def harmonic_restraint(self):

        localctx = CnsMRParser.Harmonic_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_harmonic_restraint)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 163
            self.match(CnsMRParser.Restraints)
            self.state = 164
            self.match(CnsMRParser.Harmonic)
            self.state = 165
            self.harmonic_statement()
            self.state = 166
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Rdc_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Sanisotropy(self):
            return self.getToken(CnsMRParser.Sanisotropy, 0)

        def sani_statement(self):
            return self.getTypedRuleContext(CnsMRParser.Sani_statementContext,0)


        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_rdc_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterRdc_restraint" ):
                listener.enterRdc_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitRdc_restraint" ):
                listener.exitRdc_restraint(self)




    def rdc_restraint(self):

        localctx = CnsMRParser.Rdc_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_rdc_restraint)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 168
            self.match(CnsMRParser.Sanisotropy)
            self.state = 169
            self.sani_statement()
            self.state = 170
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Coupling_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Coupling(self):
            return self.getToken(CnsMRParser.Coupling, 0)

        def coupling_statement(self):
            return self.getTypedRuleContext(CnsMRParser.Coupling_statementContext,0)


        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_coupling_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCoupling_restraint" ):
                listener.enterCoupling_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCoupling_restraint" ):
                listener.exitCoupling_restraint(self)




    def coupling_restraint(self):

        localctx = CnsMRParser.Coupling_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_coupling_restraint)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 172
            self.match(CnsMRParser.Coupling)
            self.state = 173
            self.coupling_statement()
            self.state = 174
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Carbon_shift_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Carbon(self):
            return self.getToken(CnsMRParser.Carbon, 0)

        def carbon_shift_statement(self):
            return self.getTypedRuleContext(CnsMRParser.Carbon_shift_statementContext,0)


        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_carbon_shift_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCarbon_shift_restraint" ):
                listener.enterCarbon_shift_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCarbon_shift_restraint" ):
                listener.exitCarbon_shift_restraint(self)




    def carbon_shift_restraint(self):

        localctx = CnsMRParser.Carbon_shift_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_carbon_shift_restraint)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 176
            self.match(CnsMRParser.Carbon)
            self.state = 177
            self.carbon_shift_statement()
            self.state = 178
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Proton(self):
            return self.getToken(CnsMRParser.Proton, 0)

        def proton_shift_statement(self):
            return self.getTypedRuleContext(CnsMRParser.Proton_shift_statementContext,0)


        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_restraint" ):
                listener.enterProton_shift_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_restraint" ):
                listener.exitProton_shift_restraint(self)




    def proton_shift_restraint(self):

        localctx = CnsMRParser.Proton_shift_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_proton_shift_restraint)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 180
            self.match(CnsMRParser.Proton)
            self.state = 181
            self.proton_shift_statement()
            self.state = 182
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Conformation_db_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Conformation(self):
            return self.getToken(CnsMRParser.Conformation, 0)

        def conformation_statement(self):
            return self.getTypedRuleContext(CnsMRParser.Conformation_statementContext,0)


        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_conformation_db_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterConformation_db_restraint" ):
                listener.enterConformation_db_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitConformation_db_restraint" ):
                listener.exitConformation_db_restraint(self)




    def conformation_db_restraint(self):

        localctx = CnsMRParser.Conformation_db_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_conformation_db_restraint)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 184
            self.match(CnsMRParser.Conformation)
            self.state = 185
            self.conformation_statement()
            self.state = 186
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Diffusion_anisotropy_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Danisotropy(self):
            return self.getToken(CnsMRParser.Danisotropy, 0)

        def diffusion_statement(self):
            return self.getTypedRuleContext(CnsMRParser.Diffusion_statementContext,0)


        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_diffusion_anisotropy_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDiffusion_anisotropy_restraint" ):
                listener.enterDiffusion_anisotropy_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDiffusion_anisotropy_restraint" ):
                listener.exitDiffusion_anisotropy_restraint(self)




    def diffusion_anisotropy_restraint(self):

        localctx = CnsMRParser.Diffusion_anisotropy_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_diffusion_anisotropy_restraint)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 188
            self.match(CnsMRParser.Danisotropy)
            self.state = 189
            self.diffusion_statement()
            self.state = 190
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class One_bond_coupling_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def OneBond(self):
            return self.getToken(CnsMRParser.OneBond, 0)

        def one_bond_coupling_statement(self):
            return self.getTypedRuleContext(CnsMRParser.One_bond_coupling_statementContext,0)


        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_one_bond_coupling_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterOne_bond_coupling_restraint" ):
                listener.enterOne_bond_coupling_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitOne_bond_coupling_restraint" ):
                listener.exitOne_bond_coupling_restraint(self)




    def one_bond_coupling_restraint(self):

        localctx = CnsMRParser.One_bond_coupling_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 22, self.RULE_one_bond_coupling_restraint)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 192
            self.match(CnsMRParser.OneBond)
            self.state = 193
            self.one_bond_coupling_statement()
            self.state = 194
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Angle_db_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def AngleDb(self):
            return self.getToken(CnsMRParser.AngleDb, 0)

        def angle_db_statement(self):
            return self.getTypedRuleContext(CnsMRParser.Angle_db_statementContext,0)


        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_angle_db_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAngle_db_restraint" ):
                listener.enterAngle_db_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAngle_db_restraint" ):
                listener.exitAngle_db_restraint(self)




    def angle_db_restraint(self):

        localctx = CnsMRParser.Angle_db_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 24, self.RULE_angle_db_restraint)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 196
            self.match(CnsMRParser.AngleDb)
            self.state = 197
            self.angle_db_statement()
            self.state = 198
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Noe_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Analysis(self):
            return self.getToken(CnsMRParser.Analysis, 0)

        def Equ_op(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Equ_op)
            else:
                return self.getToken(CnsMRParser.Equ_op, i)

        def Noe_analysis(self):
            return self.getToken(CnsMRParser.Noe_analysis, 0)

        def noe_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Noe_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Noe_assignContext,i)


        def Asymptote(self):
            return self.getToken(CnsMRParser.Asymptote, 0)

        def Simple_names(self):
            return self.getToken(CnsMRParser.Simple_names, 0)

        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def Averaging(self):
            return self.getToken(CnsMRParser.Averaging, 0)

        def Noe_avr_methods(self):
            return self.getToken(CnsMRParser.Noe_avr_methods, 0)

        def Bgig(self):
            return self.getToken(CnsMRParser.Bgig, 0)

        def Ceiling(self):
            return self.getToken(CnsMRParser.Ceiling, 0)

        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Simple_name(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Simple_name)
            else:
                return self.getToken(CnsMRParser.Simple_name, i)

        def CountViol(self):
            return self.getToken(CnsMRParser.CountViol, 0)

        def Cv(self):
            return self.getToken(CnsMRParser.Cv, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def Den(self):
            return self.getToken(CnsMRParser.Den, 0)

        def Initialize(self):
            return self.getToken(CnsMRParser.Initialize, 0)

        def Update(self):
            return self.getToken(CnsMRParser.Update, 0)

        def Gamma(self):
            return self.getToken(CnsMRParser.Gamma, 0)

        def Kappa(self):
            return self.getToken(CnsMRParser.Kappa, 0)

        def Distribute(self):
            return self.getToken(CnsMRParser.Distribute, 0)

        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def Ensemble(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Ensemble)
            else:
                return self.getToken(CnsMRParser.Ensemble, i)

        def Monomers(self):
            return self.getToken(CnsMRParser.Monomers, 0)

        def Ncount(self):
            return self.getToken(CnsMRParser.Ncount, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def Outd(self):
            return self.getToken(CnsMRParser.Outd, 0)

        def Partition(self):
            return self.getToken(CnsMRParser.Partition, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Noe_potential(self):
            return self.getToken(CnsMRParser.Noe_potential, 0)

        def Predict(self):
            return self.getToken(CnsMRParser.Predict, 0)

        def predict_statement(self):
            return self.getTypedRuleContext(CnsMRParser.Predict_statementContext,0)


        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def Raverage(self):
            return self.getToken(CnsMRParser.Raverage, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def Rswitch(self):
            return self.getToken(CnsMRParser.Rswitch, 0)

        def Scale(self):
            return self.getToken(CnsMRParser.Scale, 0)

        def SoExponent(self):
            return self.getToken(CnsMRParser.SoExponent, 0)

        def SqConstant(self):
            return self.getToken(CnsMRParser.SqConstant, 0)

        def SqExponent(self):
            return self.getToken(CnsMRParser.SqExponent, 0)

        def SqOffset(self):
            return self.getToken(CnsMRParser.SqOffset, 0)

        def Taverage(self):
            return self.getToken(CnsMRParser.Taverage, 0)

        def Temperature(self):
            return self.getToken(CnsMRParser.Temperature, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_noe_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNoe_statement" ):
                listener.enterNoe_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNoe_statement" ):
                listener.exitNoe_statement(self)




    def noe_statement(self):

        localctx = CnsMRParser.Noe_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 26, self.RULE_noe_statement)
        self._la = 0 # Token type
        try:
            self.state = 312
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,11,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 200
                self.match(CnsMRParser.Analysis)
                self.state = 201
                self.match(CnsMRParser.Equ_op)
                self.state = 202
                self.match(CnsMRParser.Noe_analysis)
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 206
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Assign:
                    self.state = 203
                    self.noe_assign()
                    self.state = 208
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 209
                self.match(CnsMRParser.Asymptote)
                self.state = 210
                self.match(CnsMRParser.Simple_names)
                self.state = 211
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 212
                self.match(CnsMRParser.Averaging)
                self.state = 213
                self.match(CnsMRParser.Simple_names)
                self.state = 214
                self.match(CnsMRParser.Noe_avr_methods)
                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 215
                self.match(CnsMRParser.Bgig)
                self.state = 216
                self.match(CnsMRParser.Simple_names)
                self.state = 217
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 6:
                self.enterOuterAlt(localctx, 6)
                self.state = 218
                self.match(CnsMRParser.Ceiling)
                self.state = 219
                self.match(CnsMRParser.Equ_op)
                self.state = 220
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 7:
                self.enterOuterAlt(localctx, 7)
                self.state = 221
                self.match(CnsMRParser.Classification)
                self.state = 222
                self.match(CnsMRParser.Equ_op)
                self.state = 223
                self.match(CnsMRParser.Simple_name)
                pass

            elif la_ == 8:
                self.enterOuterAlt(localctx, 8)
                self.state = 224
                self.match(CnsMRParser.CountViol)
                self.state = 225
                self.match(CnsMRParser.Simple_name)
                pass

            elif la_ == 9:
                self.enterOuterAlt(localctx, 9)
                self.state = 226
                self.match(CnsMRParser.Cv)
                self.state = 227
                self.match(CnsMRParser.Equ_op)
                self.state = 228
                self.match(CnsMRParser.Integer)
                pass

            elif la_ == 10:
                self.enterOuterAlt(localctx, 10)
                self.state = 229
                self.match(CnsMRParser.Den)
                self.state = 230
                self.match(CnsMRParser.Initialize)
                pass

            elif la_ == 11:
                self.enterOuterAlt(localctx, 11)
                self.state = 231
                self.match(CnsMRParser.Den)
                self.state = 232
                self.match(CnsMRParser.Update)
                self.state = 233
                self.match(CnsMRParser.Gamma)
                self.state = 234
                self.match(CnsMRParser.Equ_op)
                self.state = 235
                self.match(CnsMRParser.Real)
                self.state = 236
                self.match(CnsMRParser.Kappa)
                self.state = 237
                self.match(CnsMRParser.Equ_op)
                self.state = 238
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 12:
                self.enterOuterAlt(localctx, 12)
                self.state = 239
                self.match(CnsMRParser.Distribute)
                self.state = 240
                self.match(CnsMRParser.Simple_name)
                self.state = 241
                self.match(CnsMRParser.Simple_name)
                self.state = 242
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 13:
                self.enterOuterAlt(localctx, 13)
                self.state = 246
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,8,self._ctx)
                while _alt!=1 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1+1:
                        self.state = 243
                        self.match(CnsMRParser.Ensemble) 
                    self.state = 248
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,8,self._ctx)

                self.state = 249
                self.match(CnsMRParser.End)
                pass

            elif la_ == 14:
                self.enterOuterAlt(localctx, 14)
                self.state = 250
                self.match(CnsMRParser.Monomers)
                self.state = 251
                self.match(CnsMRParser.Simple_names)
                self.state = 252
                self.match(CnsMRParser.Integer)
                pass

            elif la_ == 15:
                self.enterOuterAlt(localctx, 15)
                self.state = 253
                self.match(CnsMRParser.Ncount)
                self.state = 254
                self.match(CnsMRParser.Simple_names)
                self.state = 255
                self.match(CnsMRParser.Integer)
                pass

            elif la_ == 16:
                self.enterOuterAlt(localctx, 16)
                self.state = 256
                self.match(CnsMRParser.Nrestraints)
                self.state = 257
                self.match(CnsMRParser.Equ_op)
                self.state = 258
                self.match(CnsMRParser.Integer)
                pass

            elif la_ == 17:
                self.enterOuterAlt(localctx, 17)
                self.state = 259
                self.match(CnsMRParser.Outd)
                pass

            elif la_ == 18:
                self.enterOuterAlt(localctx, 18)
                self.state = 260
                self.match(CnsMRParser.Partition)
                self.state = 261
                self.match(CnsMRParser.Equ_op)
                self.state = 262
                self.match(CnsMRParser.Integer)
                pass

            elif la_ == 19:
                self.enterOuterAlt(localctx, 19)
                self.state = 263
                self.match(CnsMRParser.Potential)
                self.state = 264
                self.match(CnsMRParser.Simple_names)
                self.state = 265
                self.match(CnsMRParser.Noe_potential)
                pass

            elif la_ == 20:
                self.enterOuterAlt(localctx, 20)
                self.state = 266
                self.match(CnsMRParser.Predict)
                self.state = 267
                self.predict_statement()
                self.state = 268
                self.match(CnsMRParser.End)
                pass

            elif la_ == 21:
                self.enterOuterAlt(localctx, 21)
                self.state = 270
                self.match(CnsMRParser.Print)
                self.state = 271
                self.match(CnsMRParser.Threshold)
                self.state = 272
                self.match(CnsMRParser.Equ_op)
                self.state = 273
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 22:
                self.enterOuterAlt(localctx, 22)
                self.state = 274
                self.match(CnsMRParser.Raverage)
                self.state = 278
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,9,self._ctx)
                while _alt!=1 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1+1:
                        self.state = 275
                        self.match(CnsMRParser.Simple_name) 
                    self.state = 280
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,9,self._ctx)

                self.state = 281
                self.match(CnsMRParser.End)
                pass

            elif la_ == 23:
                self.enterOuterAlt(localctx, 23)
                self.state = 282
                self.match(CnsMRParser.Reset)
                pass

            elif la_ == 24:
                self.enterOuterAlt(localctx, 24)
                self.state = 283
                self.match(CnsMRParser.Rswitch)
                self.state = 284
                self.match(CnsMRParser.Simple_names)
                self.state = 285
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 25:
                self.enterOuterAlt(localctx, 25)
                self.state = 286
                self.match(CnsMRParser.Scale)
                self.state = 287
                self.match(CnsMRParser.Simple_names)
                self.state = 288
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 26:
                self.enterOuterAlt(localctx, 26)
                self.state = 289
                self.match(CnsMRParser.SoExponent)
                self.state = 290
                self.match(CnsMRParser.Simple_names)
                self.state = 291
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 27:
                self.enterOuterAlt(localctx, 27)
                self.state = 292
                self.match(CnsMRParser.SqConstant)
                self.state = 293
                self.match(CnsMRParser.Simple_names)
                self.state = 294
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 28:
                self.enterOuterAlt(localctx, 28)
                self.state = 295
                self.match(CnsMRParser.SqExponent)
                self.state = 296
                self.match(CnsMRParser.Simple_names)
                self.state = 297
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 29:
                self.enterOuterAlt(localctx, 29)
                self.state = 298
                self.match(CnsMRParser.SqOffset)
                self.state = 299
                self.match(CnsMRParser.Simple_names)
                self.state = 300
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 30:
                self.enterOuterAlt(localctx, 30)
                self.state = 301
                self.match(CnsMRParser.Taverage)
                self.state = 305
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,10,self._ctx)
                while _alt!=1 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1+1:
                        self.state = 302
                        self.match(CnsMRParser.Simple_name) 
                    self.state = 307
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,10,self._ctx)

                self.state = 308
                self.match(CnsMRParser.End)
                pass

            elif la_ == 31:
                self.enterOuterAlt(localctx, 31)
                self.state = 309
                self.match(CnsMRParser.Temperature)
                self.state = 310
                self.match(CnsMRParser.Equ_op)
                self.state = 311
                self.match(CnsMRParser.Real)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Noe_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def Or_op(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Or_op)
            else:
                return self.getToken(CnsMRParser.Or_op, i)

        def getRuleIndex(self):
            return CnsMRParser.RULE_noe_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNoe_assign" ):
                listener.enterNoe_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNoe_assign" ):
                listener.exitNoe_assign(self)




    def noe_assign(self):

        localctx = CnsMRParser.Noe_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 28, self.RULE_noe_assign)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 314
            self.match(CnsMRParser.Assign)
            self.state = 315
            self.selection()
            self.state = 316
            self.selection()
            self.state = 317
            _la = self._input.LA(1)
            if not(_la==CnsMRParser.Integer or _la==CnsMRParser.Real):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 318
            self.match(CnsMRParser.Real)
            self.state = 319
            self.match(CnsMRParser.Real)
            self.state = 326
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==CnsMRParser.Or_op:
                self.state = 320
                self.match(CnsMRParser.Or_op)
                self.state = 321
                self.selection()
                self.state = 322
                self.selection()
                self.state = 328
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Predict_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Cutoff(self):
            return self.getToken(CnsMRParser.Cutoff, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Real(self):
            return self.getToken(CnsMRParser.Real, 0)

        def Cuton(self):
            return self.getToken(CnsMRParser.Cuton, 0)

        def From(self):
            return self.getToken(CnsMRParser.From, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def To(self):
            return self.getToken(CnsMRParser.To, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_predict_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPredict_statement" ):
                listener.enterPredict_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPredict_statement" ):
                listener.exitPredict_statement(self)




    def predict_statement(self):

        localctx = CnsMRParser.Predict_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 30, self.RULE_predict_statement)
        try:
            self.state = 339
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Cutoff]:
                self.enterOuterAlt(localctx, 1)
                self.state = 329
                self.match(CnsMRParser.Cutoff)
                self.state = 330
                self.match(CnsMRParser.Equ_op)
                self.state = 331
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.Cuton]:
                self.enterOuterAlt(localctx, 2)
                self.state = 332
                self.match(CnsMRParser.Cuton)
                self.state = 333
                self.match(CnsMRParser.Equ_op)
                self.state = 334
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.From]:
                self.enterOuterAlt(localctx, 3)
                self.state = 335
                self.match(CnsMRParser.From)
                self.state = 336
                self.selection()
                pass
            elif token in [CnsMRParser.To]:
                self.enterOuterAlt(localctx, 4)
                self.state = 337
                self.match(CnsMRParser.To)
                self.state = 338
                self.selection()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dihedral_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def dihedral_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Dihedral_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Dihedral_assignContext,i)


        def Cv(self):
            return self.getToken(CnsMRParser.Cv, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def Nassign(self):
            return self.getToken(CnsMRParser.Nassign, 0)

        def Partition(self):
            return self.getToken(CnsMRParser.Partition, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def Scale(self):
            return self.getToken(CnsMRParser.Scale, 0)

        def Real(self):
            return self.getToken(CnsMRParser.Real, 0)

        def Print_any(self):
            return self.getToken(CnsMRParser.Print_any, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_dihedral_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDihedral_statement" ):
                listener.enterDihedral_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDihedral_statement" ):
                listener.exitDihedral_statement(self)




    def dihedral_statement(self):

        localctx = CnsMRParser.Dihedral_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 32, self.RULE_dihedral_statement)
        self._la = 0 # Token type
        try:
            self.state = 361
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.End, CnsMRParser.Assign]:
                self.enterOuterAlt(localctx, 1)
                self.state = 344
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Assign:
                    self.state = 341
                    self.dihedral_assign()
                    self.state = 346
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [CnsMRParser.Cv]:
                self.enterOuterAlt(localctx, 2)
                self.state = 347
                self.match(CnsMRParser.Cv)
                self.state = 348
                self.match(CnsMRParser.Equ_op)
                self.state = 349
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Nassign]:
                self.enterOuterAlt(localctx, 3)
                self.state = 350
                self.match(CnsMRParser.Nassign)
                self.state = 351
                self.match(CnsMRParser.Equ_op)
                self.state = 352
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Partition]:
                self.enterOuterAlt(localctx, 4)
                self.state = 353
                self.match(CnsMRParser.Partition)
                self.state = 354
                self.match(CnsMRParser.Equ_op)
                self.state = 355
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 5)
                self.state = 356
                self.match(CnsMRParser.Reset)
                pass
            elif token in [CnsMRParser.Scale]:
                self.enterOuterAlt(localctx, 6)
                self.state = 357
                self.match(CnsMRParser.Scale)
                self.state = 358
                self.match(CnsMRParser.Equ_op)
                self.state = 359
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.Print_any]:
                self.enterOuterAlt(localctx, 7)
                self.state = 360
                self.match(CnsMRParser.Print_any)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dihedral_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Integer)
            else:
                return self.getToken(CnsMRParser.Integer, i)

        def getRuleIndex(self):
            return CnsMRParser.RULE_dihedral_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDihedral_assign" ):
                listener.enterDihedral_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDihedral_assign" ):
                listener.exitDihedral_assign(self)




    def dihedral_assign(self):

        localctx = CnsMRParser.Dihedral_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 34, self.RULE_dihedral_assign)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 363
            self.match(CnsMRParser.Assign)
            self.state = 364
            self.selection()
            self.state = 365
            self.selection()
            self.state = 366
            self.selection()
            self.state = 367
            self.selection()
            self.state = 368
            _la = self._input.LA(1)
            if not(_la==CnsMRParser.Integer or _la==CnsMRParser.Real):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 369
            self.match(CnsMRParser.Real)
            self.state = 370
            self.match(CnsMRParser.Real)
            self.state = 371
            self.match(CnsMRParser.Integer)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Plane_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Group(self):
            return self.getToken(CnsMRParser.Group, 0)

        def group_statement(self):
            return self.getTypedRuleContext(CnsMRParser.Group_statementContext,0)


        def Initialize(self):
            return self.getToken(CnsMRParser.Initialize, 0)

        def Print_any(self):
            return self.getToken(CnsMRParser.Print_any, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_plane_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPlane_statement" ):
                listener.enterPlane_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPlane_statement" ):
                listener.exitPlane_statement(self)




    def plane_statement(self):

        localctx = CnsMRParser.Plane_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 36, self.RULE_plane_statement)
        try:
            self.state = 377
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Group]:
                self.enterOuterAlt(localctx, 1)
                self.state = 373
                self.match(CnsMRParser.Group)
                self.state = 374
                self.group_statement()
                pass
            elif token in [CnsMRParser.Initialize]:
                self.enterOuterAlt(localctx, 2)
                self.state = 375
                self.match(CnsMRParser.Initialize)
                pass
            elif token in [CnsMRParser.Print_any]:
                self.enterOuterAlt(localctx, 3)
                self.state = 376
                self.match(CnsMRParser.Print_any)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Group_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Selection(self):
            return self.getToken(CnsMRParser.Selection, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def Weight(self):
            return self.getToken(CnsMRParser.Weight, 0)

        def Real(self):
            return self.getToken(CnsMRParser.Real, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_group_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterGroup_statement" ):
                listener.enterGroup_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitGroup_statement" ):
                listener.exitGroup_statement(self)




    def group_statement(self):

        localctx = CnsMRParser.Group_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 38, self.RULE_group_statement)
        self._la = 0 # Token type
        try:
            self.state = 385
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Selection]:
                self.enterOuterAlt(localctx, 1)
                self.state = 379
                self.match(CnsMRParser.Selection)
                self.state = 380
                self.match(CnsMRParser.Equ_op)
                self.state = 381
                self.selection()
                pass
            elif token in [CnsMRParser.Weight]:
                self.enterOuterAlt(localctx, 2)
                self.state = 382
                self.match(CnsMRParser.Weight)
                self.state = 383
                self.match(CnsMRParser.Equ_op)
                self.state = 384
                _la = self._input.LA(1)
                if not(_la==CnsMRParser.Integer or _la==CnsMRParser.Real):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Harmonic_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Exponent(self):
            return self.getToken(CnsMRParser.Exponent, 0)

        def Equ_op(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Equ_op)
            else:
                return self.getToken(CnsMRParser.Equ_op, i)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def Normal(self):
            return self.getToken(CnsMRParser.Normal, 0)

        def L_paren(self):
            return self.getToken(CnsMRParser.L_paren, 0)

        def R_paren(self):
            return self.getToken(CnsMRParser.R_paren, 0)

        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def Tail(self):
            return self.getToken(CnsMRParser.Tail, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def Comma(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Comma)
            else:
                return self.getToken(CnsMRParser.Comma, i)

        def Head(self):
            return self.getToken(CnsMRParser.Head, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_harmonic_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterHarmonic_statement" ):
                listener.enterHarmonic_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitHarmonic_statement" ):
                listener.exitHarmonic_statement(self)




    def harmonic_statement(self):

        localctx = CnsMRParser.Harmonic_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 40, self.RULE_harmonic_statement)
        self._la = 0 # Token type
        try:
            self.state = 416
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Exponent]:
                self.enterOuterAlt(localctx, 1)
                self.state = 387
                self.match(CnsMRParser.Exponent)
                self.state = 388
                self.match(CnsMRParser.Equ_op)
                self.state = 389
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Normal]:
                self.enterOuterAlt(localctx, 2)
                self.state = 390
                self.match(CnsMRParser.Normal)
                self.state = 391
                self.match(CnsMRParser.Equ_op)
                self.state = 392
                self.match(CnsMRParser.L_paren)
                self.state = 413
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.Real]:
                    self.state = 393
                    self.match(CnsMRParser.Real)
                    self.state = 395
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==CnsMRParser.Comma:
                        self.state = 394
                        self.match(CnsMRParser.Comma)


                    self.state = 397
                    self.match(CnsMRParser.Real)
                    self.state = 399
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==CnsMRParser.Comma:
                        self.state = 398
                        self.match(CnsMRParser.Comma)


                    self.state = 401
                    self.match(CnsMRParser.Real)
                    pass
                elif token in [CnsMRParser.Tail]:
                    self.state = 402
                    self.match(CnsMRParser.Tail)
                    self.state = 403
                    self.match(CnsMRParser.Equ_op)
                    self.state = 404
                    self.selection()
                    self.state = 406
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==CnsMRParser.Comma:
                        self.state = 405
                        self.match(CnsMRParser.Comma)


                    self.state = 411
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==CnsMRParser.Head:
                        self.state = 408
                        self.match(CnsMRParser.Head)
                        self.state = 409
                        self.match(CnsMRParser.Equ_op)
                        self.state = 410
                        self.selection()


                    pass
                else:
                    raise NoViableAltException(self)

                self.state = 415
                self.match(CnsMRParser.R_paren)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Sani_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def sani_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Sani_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Sani_assignContext,i)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def Coefficients(self):
            return self.getToken(CnsMRParser.Coefficients, 0)

        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Rdc_potential(self):
            return self.getToken(CnsMRParser.Rdc_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_sani_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSani_statement" ):
                listener.enterSani_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSani_statement" ):
                listener.exitSani_statement(self)




    def sani_statement(self):

        localctx = CnsMRParser.Sani_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 42, self.RULE_sani_statement)
        self._la = 0 # Token type
        try:
            self.state = 444
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.End, CnsMRParser.Assign]:
                self.enterOuterAlt(localctx, 1)
                self.state = 421
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Assign:
                    self.state = 418
                    self.sani_assign()
                    self.state = 423
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [CnsMRParser.Classification]:
                self.enterOuterAlt(localctx, 2)
                self.state = 424
                self.match(CnsMRParser.Classification)
                self.state = 425
                self.match(CnsMRParser.Equ_op)
                self.state = 426
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.Coefficients]:
                self.enterOuterAlt(localctx, 3)
                self.state = 427
                self.match(CnsMRParser.Coefficients)
                self.state = 428
                self.match(CnsMRParser.Real)
                self.state = 429
                self.match(CnsMRParser.Real)
                self.state = 430
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.ForceConstant]:
                self.enterOuterAlt(localctx, 4)
                self.state = 431
                self.match(CnsMRParser.ForceConstant)
                self.state = 432
                self.match(CnsMRParser.Equ_op)
                self.state = 433
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.Nrestraints]:
                self.enterOuterAlt(localctx, 5)
                self.state = 434
                self.match(CnsMRParser.Nrestraints)
                self.state = 435
                self.match(CnsMRParser.Equ_op)
                self.state = 436
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Potential]:
                self.enterOuterAlt(localctx, 6)
                self.state = 437
                self.match(CnsMRParser.Potential)
                self.state = 438
                self.match(CnsMRParser.Equ_op)
                self.state = 439
                self.match(CnsMRParser.Rdc_potential)
                pass
            elif token in [CnsMRParser.Print]:
                self.enterOuterAlt(localctx, 7)
                self.state = 440
                self.match(CnsMRParser.Print)
                self.state = 441
                self.match(CnsMRParser.Threshold)
                self.state = 442
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 8)
                self.state = 443
                self.match(CnsMRParser.Reset)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Sani_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_sani_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSani_assign" ):
                listener.enterSani_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSani_assign" ):
                listener.exitSani_assign(self)




    def sani_assign(self):

        localctx = CnsMRParser.Sani_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 44, self.RULE_sani_assign)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 446
            self.match(CnsMRParser.Assign)
            self.state = 447
            self.selection()
            self.state = 448
            self.selection()
            self.state = 449
            self.selection()
            self.state = 450
            self.selection()
            self.state = 451
            self.selection()
            self.state = 452
            self.selection()
            self.state = 453
            _la = self._input.LA(1)
            if not(_la==CnsMRParser.Integer or _la==CnsMRParser.Real):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 454
            self.match(CnsMRParser.Real)
            self.state = 456
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.Real:
                self.state = 455
                self.match(CnsMRParser.Real)


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Coupling_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def coup_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Coup_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Coup_assignContext,i)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def Coefficients(self):
            return self.getToken(CnsMRParser.Coefficients, 0)

        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def Cv(self):
            return self.getToken(CnsMRParser.Cv, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def Partition(self):
            return self.getToken(CnsMRParser.Partition, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Coupling_potential(self):
            return self.getToken(CnsMRParser.Coupling_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def All(self):
            return self.getToken(CnsMRParser.All, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_coupling_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCoupling_statement" ):
                listener.enterCoupling_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCoupling_statement" ):
                listener.exitCoupling_statement(self)




    def coupling_statement(self):

        localctx = CnsMRParser.Coupling_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 46, self.RULE_coupling_statement)
        self._la = 0 # Token type
        try:
            self.state = 499
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.End, CnsMRParser.Assign]:
                self.enterOuterAlt(localctx, 1)
                self.state = 461
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Assign:
                    self.state = 458
                    self.coup_assign()
                    self.state = 463
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [CnsMRParser.Classification]:
                self.enterOuterAlt(localctx, 2)
                self.state = 464
                self.match(CnsMRParser.Classification)
                self.state = 465
                self.match(CnsMRParser.Equ_op)
                self.state = 466
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.Coefficients]:
                self.enterOuterAlt(localctx, 3)
                self.state = 467
                self.match(CnsMRParser.Coefficients)
                self.state = 468
                self.match(CnsMRParser.Real)
                self.state = 469
                self.match(CnsMRParser.Real)
                self.state = 470
                self.match(CnsMRParser.Real)
                self.state = 471
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.Cv]:
                self.enterOuterAlt(localctx, 4)
                self.state = 472
                self.match(CnsMRParser.Cv)
                self.state = 473
                self.match(CnsMRParser.Equ_op)
                self.state = 474
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.ForceConstant]:
                self.enterOuterAlt(localctx, 5)
                self.state = 475
                self.match(CnsMRParser.ForceConstant)
                self.state = 476
                self.match(CnsMRParser.Real)
                self.state = 478
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Real:
                    self.state = 477
                    self.match(CnsMRParser.Real)


                pass
            elif token in [CnsMRParser.Nrestraints]:
                self.enterOuterAlt(localctx, 6)
                self.state = 480
                self.match(CnsMRParser.Nrestraints)
                self.state = 481
                self.match(CnsMRParser.Equ_op)
                self.state = 482
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Partition]:
                self.enterOuterAlt(localctx, 7)
                self.state = 483
                self.match(CnsMRParser.Partition)
                self.state = 484
                self.match(CnsMRParser.Equ_op)
                self.state = 485
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Potential]:
                self.enterOuterAlt(localctx, 8)
                self.state = 486
                self.match(CnsMRParser.Potential)
                self.state = 487
                self.match(CnsMRParser.Equ_op)
                self.state = 488
                self.match(CnsMRParser.Coupling_potential)
                pass
            elif token in [CnsMRParser.Print]:
                self.enterOuterAlt(localctx, 9)
                self.state = 489
                self.match(CnsMRParser.Print)
                self.state = 490
                self.match(CnsMRParser.Threshold)
                self.state = 491
                self.match(CnsMRParser.Real)
                self.state = 496
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.All]:
                    self.state = 492
                    self.match(CnsMRParser.All)
                    pass
                elif token in [CnsMRParser.Classification]:
                    self.state = 493
                    self.match(CnsMRParser.Classification)
                    self.state = 494
                    self.match(CnsMRParser.Equ_op)
                    self.state = 495
                    self.match(CnsMRParser.Simple_name)
                    pass
                else:
                    raise NoViableAltException(self)

                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 10)
                self.state = 498
                self.match(CnsMRParser.Reset)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Coup_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def getRuleIndex(self):
            return CnsMRParser.RULE_coup_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCoup_assign" ):
                listener.enterCoup_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCoup_assign" ):
                listener.exitCoup_assign(self)




    def coup_assign(self):

        localctx = CnsMRParser.Coup_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 48, self.RULE_coup_assign)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 501
            self.match(CnsMRParser.Assign)
            self.state = 502
            self.selection()
            self.state = 503
            self.selection()
            self.state = 504
            self.selection()
            self.state = 505
            self.selection()
            self.state = 511
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.L_paren:
                self.state = 506
                self.selection()
                self.state = 507
                self.selection()
                self.state = 508
                self.selection()
                self.state = 509
                self.selection()


            self.state = 513
            self.match(CnsMRParser.Real)
            self.state = 514
            self.match(CnsMRParser.Real)
            self.state = 517
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.Real:
                self.state = 515
                self.match(CnsMRParser.Real)
                self.state = 516
                self.match(CnsMRParser.Real)


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Carbon_shift_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def carbon_shift_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Carbon_shift_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Carbon_shift_assignContext,i)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def Expectation(self):
            return self.getToken(CnsMRParser.Expectation, 0)

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Integer)
            else:
                return self.getToken(CnsMRParser.Integer, i)

        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def PhiStep(self):
            return self.getToken(CnsMRParser.PhiStep, 0)

        def PsiStep(self):
            return self.getToken(CnsMRParser.PsiStep, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Rdc_potential(self):
            return self.getToken(CnsMRParser.Rdc_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def carbon_shift_rcoil(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Carbon_shift_rcoilContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Carbon_shift_rcoilContext,i)


        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def Zero(self):
            return self.getToken(CnsMRParser.Zero, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_carbon_shift_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCarbon_shift_statement" ):
                listener.enterCarbon_shift_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCarbon_shift_statement" ):
                listener.exitCarbon_shift_statement(self)




    def carbon_shift_statement(self):

        localctx = CnsMRParser.Carbon_shift_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 50, self.RULE_carbon_shift_statement)
        self._la = 0 # Token type
        try:
            self.state = 560
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,35,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 522
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Assign:
                    self.state = 519
                    self.carbon_shift_assign()
                    self.state = 524
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 525
                self.match(CnsMRParser.Classification)
                self.state = 526
                self.match(CnsMRParser.Equ_op)
                self.state = 527
                self.match(CnsMRParser.Simple_name)
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 528
                self.match(CnsMRParser.Expectation)
                self.state = 529
                self.match(CnsMRParser.Integer)
                self.state = 530
                self.match(CnsMRParser.Integer)
                self.state = 531
                self.match(CnsMRParser.Real)
                self.state = 532
                self.match(CnsMRParser.Real)
                self.state = 533
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 534
                self.match(CnsMRParser.ForceConstant)
                self.state = 535
                self.match(CnsMRParser.Equ_op)
                self.state = 536
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 537
                self.match(CnsMRParser.Nrestraints)
                self.state = 538
                self.match(CnsMRParser.Equ_op)
                self.state = 539
                self.match(CnsMRParser.Integer)
                pass

            elif la_ == 6:
                self.enterOuterAlt(localctx, 6)
                self.state = 540
                self.match(CnsMRParser.PhiStep)
                self.state = 541
                self.match(CnsMRParser.Equ_op)
                self.state = 542
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 7:
                self.enterOuterAlt(localctx, 7)
                self.state = 543
                self.match(CnsMRParser.PsiStep)
                self.state = 544
                self.match(CnsMRParser.Equ_op)
                self.state = 545
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 8:
                self.enterOuterAlt(localctx, 8)
                self.state = 546
                self.match(CnsMRParser.Potential)
                self.state = 547
                self.match(CnsMRParser.Equ_op)
                self.state = 548
                self.match(CnsMRParser.Rdc_potential)
                pass

            elif la_ == 9:
                self.enterOuterAlt(localctx, 9)
                self.state = 549
                self.match(CnsMRParser.Print)
                self.state = 550
                self.match(CnsMRParser.Threshold)
                self.state = 551
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 10:
                self.enterOuterAlt(localctx, 10)
                self.state = 555
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Rcoil:
                    self.state = 552
                    self.carbon_shift_rcoil()
                    self.state = 557
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 11:
                self.enterOuterAlt(localctx, 11)
                self.state = 558
                self.match(CnsMRParser.Reset)
                pass

            elif la_ == 12:
                self.enterOuterAlt(localctx, 12)
                self.state = 559
                self.match(CnsMRParser.Zero)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Carbon_shift_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def getRuleIndex(self):
            return CnsMRParser.RULE_carbon_shift_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCarbon_shift_assign" ):
                listener.enterCarbon_shift_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCarbon_shift_assign" ):
                listener.exitCarbon_shift_assign(self)




    def carbon_shift_assign(self):

        localctx = CnsMRParser.Carbon_shift_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 52, self.RULE_carbon_shift_assign)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 562
            self.match(CnsMRParser.Assign)
            self.state = 563
            self.selection()
            self.state = 564
            self.selection()
            self.state = 565
            self.selection()
            self.state = 566
            self.selection()
            self.state = 567
            self.selection()
            self.state = 568
            self.match(CnsMRParser.Real)
            self.state = 569
            self.match(CnsMRParser.Real)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Carbon_shift_rcoilContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Rcoil(self):
            return self.getToken(CnsMRParser.Rcoil, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def getRuleIndex(self):
            return CnsMRParser.RULE_carbon_shift_rcoil

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCarbon_shift_rcoil" ):
                listener.enterCarbon_shift_rcoil(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCarbon_shift_rcoil" ):
                listener.exitCarbon_shift_rcoil(self)




    def carbon_shift_rcoil(self):

        localctx = CnsMRParser.Carbon_shift_rcoilContext(self, self._ctx, self.state)
        self.enterRule(localctx, 54, self.RULE_carbon_shift_rcoil)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 571
            self.match(CnsMRParser.Rcoil)
            self.state = 572
            self.selection()
            self.state = 573
            self.match(CnsMRParser.Real)
            self.state = 574
            self.match(CnsMRParser.Real)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def observed(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.ObservedContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.ObservedContext,i)


        def proton_shift_rcoil(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Proton_shift_rcoilContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Proton_shift_rcoilContext,i)


        def proton_shift_anisotropy(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Proton_shift_anisotropyContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Proton_shift_anisotropyContext,i)


        def proton_shift_amides(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Proton_shift_amidesContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Proton_shift_amidesContext,i)


        def proton_shift_carbons(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Proton_shift_carbonsContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Proton_shift_carbonsContext,i)


        def proton_shift_nitrogens(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Proton_shift_nitrogensContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Proton_shift_nitrogensContext,i)


        def proton_shift_oxygens(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Proton_shift_oxygensContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Proton_shift_oxygensContext,i)


        def proton_shift_ring_atoms(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Proton_shift_ring_atomsContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Proton_shift_ring_atomsContext,i)


        def proton_shift_alphas_and_amides(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Proton_shift_alphas_and_amidesContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Proton_shift_alphas_and_amidesContext,i)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Simple_name(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Simple_name)
            else:
                return self.getToken(CnsMRParser.Simple_name, i)

        def Error(self):
            return self.getToken(CnsMRParser.Error, 0)

        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Coupling_potential(self):
            return self.getToken(CnsMRParser.Coupling_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def All(self):
            return self.getToken(CnsMRParser.All, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_statement" ):
                listener.enterProton_shift_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_statement" ):
                listener.exitProton_shift_statement(self)




    def proton_shift_statement(self):

        localctx = CnsMRParser.Proton_shift_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 56, self.RULE_proton_shift_statement)
        self._la = 0 # Token type
        try:
            self.state = 656
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,48,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 579
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Observed:
                    self.state = 576
                    self.observed()
                    self.state = 581
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 585
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Rcoil:
                    self.state = 582
                    self.proton_shift_rcoil()
                    self.state = 587
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 591
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Anisotropy:
                    self.state = 588
                    self.proton_shift_anisotropy()
                    self.state = 593
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 597
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Amides:
                    self.state = 594
                    self.proton_shift_amides()
                    self.state = 599
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 603
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Carbons:
                    self.state = 600
                    self.proton_shift_carbons()
                    self.state = 605
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 6:
                self.enterOuterAlt(localctx, 6)
                self.state = 609
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Nitrogens:
                    self.state = 606
                    self.proton_shift_nitrogens()
                    self.state = 611
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 7:
                self.enterOuterAlt(localctx, 7)
                self.state = 615
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Oxygens:
                    self.state = 612
                    self.proton_shift_oxygens()
                    self.state = 617
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 8:
                self.enterOuterAlt(localctx, 8)
                self.state = 621
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.RingAtoms:
                    self.state = 618
                    self.proton_shift_ring_atoms()
                    self.state = 623
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 9:
                self.enterOuterAlt(localctx, 9)
                self.state = 627
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.AlphasAndAmides:
                    self.state = 624
                    self.proton_shift_alphas_and_amides()
                    self.state = 629
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass

            elif la_ == 10:
                self.enterOuterAlt(localctx, 10)
                self.state = 630
                self.match(CnsMRParser.Classification)
                self.state = 631
                self.match(CnsMRParser.Equ_op)
                self.state = 632
                self.match(CnsMRParser.Simple_name)
                pass

            elif la_ == 11:
                self.enterOuterAlt(localctx, 11)
                self.state = 633
                self.match(CnsMRParser.Error)
                self.state = 635
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 634
                    self.match(CnsMRParser.Equ_op)


                self.state = 637
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 12:
                self.enterOuterAlt(localctx, 12)
                self.state = 638
                self.match(CnsMRParser.ForceConstant)
                self.state = 639
                self.match(CnsMRParser.Real)
                self.state = 641
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Real:
                    self.state = 640
                    self.match(CnsMRParser.Real)


                pass

            elif la_ == 13:
                self.enterOuterAlt(localctx, 13)
                self.state = 643
                self.match(CnsMRParser.Potential)
                self.state = 644
                self.match(CnsMRParser.Coupling_potential)
                pass

            elif la_ == 14:
                self.enterOuterAlt(localctx, 14)
                self.state = 645
                self.match(CnsMRParser.Print)
                self.state = 646
                self.match(CnsMRParser.Threshold)
                self.state = 647
                self.match(CnsMRParser.Real)
                self.state = 652
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.All]:
                    self.state = 648
                    self.match(CnsMRParser.All)
                    pass
                elif token in [CnsMRParser.Classification]:
                    self.state = 649
                    self.match(CnsMRParser.Classification)
                    self.state = 650
                    self.match(CnsMRParser.Equ_op)
                    self.state = 651
                    self.match(CnsMRParser.Simple_name)
                    pass
                else:
                    raise NoViableAltException(self)

                self.state = 654
                self.match(CnsMRParser.Simple_name)
                pass

            elif la_ == 15:
                self.enterOuterAlt(localctx, 15)
                self.state = 655
                self.match(CnsMRParser.Reset)
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ObservedContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Observed(self):
            return self.getToken(CnsMRParser.Observed, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def getRuleIndex(self):
            return CnsMRParser.RULE_observed

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterObserved" ):
                listener.enterObserved(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitObserved" ):
                listener.exitObserved(self)




    def observed(self):

        localctx = CnsMRParser.ObservedContext(self, self._ctx, self.state)
        self.enterRule(localctx, 58, self.RULE_observed)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 658
            self.match(CnsMRParser.Observed)
            self.state = 659
            self.selection()
            self.state = 661
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.L_paren:
                self.state = 660
                self.selection()


            self.state = 663
            self.match(CnsMRParser.Real)
            self.state = 665
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.Real:
                self.state = 664
                self.match(CnsMRParser.Real)


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_rcoilContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Rcoil(self):
            return self.getToken(CnsMRParser.Rcoil, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def Real(self):
            return self.getToken(CnsMRParser.Real, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_rcoil

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_rcoil" ):
                listener.enterProton_shift_rcoil(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_rcoil" ):
                listener.exitProton_shift_rcoil(self)




    def proton_shift_rcoil(self):

        localctx = CnsMRParser.Proton_shift_rcoilContext(self, self._ctx, self.state)
        self.enterRule(localctx, 60, self.RULE_proton_shift_rcoil)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 667
            self.match(CnsMRParser.Rcoil)
            self.state = 668
            self.selection()
            self.state = 669
            self.match(CnsMRParser.Real)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_anisotropyContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Anisotropy(self):
            return self.getToken(CnsMRParser.Anisotropy, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def Simple_name(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Simple_name)
            else:
                return self.getToken(CnsMRParser.Simple_name, i)

        def Logical(self):
            return self.getToken(CnsMRParser.Logical, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_anisotropy

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_anisotropy" ):
                listener.enterProton_shift_anisotropy(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_anisotropy" ):
                listener.exitProton_shift_anisotropy(self)




    def proton_shift_anisotropy(self):

        localctx = CnsMRParser.Proton_shift_anisotropyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 62, self.RULE_proton_shift_anisotropy)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 671
            self.match(CnsMRParser.Anisotropy)
            self.state = 672
            self.selection()
            self.state = 673
            self.selection()
            self.state = 674
            self.selection()
            self.state = 675
            self.match(CnsMRParser.Simple_name)
            self.state = 677
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.Logical:
                self.state = 676
                self.match(CnsMRParser.Logical)


            self.state = 679
            self.match(CnsMRParser.Simple_name)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_amidesContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Amides(self):
            return self.getToken(CnsMRParser.Amides, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_amides

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_amides" ):
                listener.enterProton_shift_amides(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_amides" ):
                listener.exitProton_shift_amides(self)




    def proton_shift_amides(self):

        localctx = CnsMRParser.Proton_shift_amidesContext(self, self._ctx, self.state)
        self.enterRule(localctx, 64, self.RULE_proton_shift_amides)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 681
            self.match(CnsMRParser.Amides)
            self.state = 682
            self.selection()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_carbonsContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Carbons(self):
            return self.getToken(CnsMRParser.Carbons, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_carbons

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_carbons" ):
                listener.enterProton_shift_carbons(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_carbons" ):
                listener.exitProton_shift_carbons(self)




    def proton_shift_carbons(self):

        localctx = CnsMRParser.Proton_shift_carbonsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 66, self.RULE_proton_shift_carbons)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 684
            self.match(CnsMRParser.Carbons)
            self.state = 685
            self.selection()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_nitrogensContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Nitrogens(self):
            return self.getToken(CnsMRParser.Nitrogens, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_nitrogens

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_nitrogens" ):
                listener.enterProton_shift_nitrogens(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_nitrogens" ):
                listener.exitProton_shift_nitrogens(self)




    def proton_shift_nitrogens(self):

        localctx = CnsMRParser.Proton_shift_nitrogensContext(self, self._ctx, self.state)
        self.enterRule(localctx, 68, self.RULE_proton_shift_nitrogens)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 687
            self.match(CnsMRParser.Nitrogens)
            self.state = 688
            self.selection()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_oxygensContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Oxygens(self):
            return self.getToken(CnsMRParser.Oxygens, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_oxygens

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_oxygens" ):
                listener.enterProton_shift_oxygens(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_oxygens" ):
                listener.exitProton_shift_oxygens(self)




    def proton_shift_oxygens(self):

        localctx = CnsMRParser.Proton_shift_oxygensContext(self, self._ctx, self.state)
        self.enterRule(localctx, 70, self.RULE_proton_shift_oxygens)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 690
            self.match(CnsMRParser.Oxygens)
            self.state = 691
            self.selection()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_ring_atomsContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def RingAtoms(self):
            return self.getToken(CnsMRParser.RingAtoms, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_ring_atoms

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_ring_atoms" ):
                listener.enterProton_shift_ring_atoms(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_ring_atoms" ):
                listener.exitProton_shift_ring_atoms(self)




    def proton_shift_ring_atoms(self):

        localctx = CnsMRParser.Proton_shift_ring_atomsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 72, self.RULE_proton_shift_ring_atoms)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 693
            self.match(CnsMRParser.RingAtoms)
            self.state = 694
            self.match(CnsMRParser.Simple_name)
            self.state = 695
            self.selection()
            self.state = 696
            self.selection()
            self.state = 697
            self.selection()
            self.state = 698
            self.selection()
            self.state = 699
            self.selection()
            self.state = 701
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.L_paren:
                self.state = 700
                self.selection()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_alphas_and_amidesContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def AlphasAndAmides(self):
            return self.getToken(CnsMRParser.AlphasAndAmides, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_alphas_and_amides

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_alphas_and_amides" ):
                listener.enterProton_shift_alphas_and_amides(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_alphas_and_amides" ):
                listener.exitProton_shift_alphas_and_amides(self)




    def proton_shift_alphas_and_amides(self):

        localctx = CnsMRParser.Proton_shift_alphas_and_amidesContext(self, self._ctx, self.state)
        self.enterRule(localctx, 74, self.RULE_proton_shift_alphas_and_amides)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 703
            self.match(CnsMRParser.AlphasAndAmides)
            self.state = 704
            self.selection()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Conformation_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def conf_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Conf_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Conf_assignContext,i)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def Compressed(self):
            return self.getToken(CnsMRParser.Compressed, 0)

        def Expectation(self):
            return self.getToken(CnsMRParser.Expectation, 0)

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Integer)
            else:
                return self.getToken(CnsMRParser.Integer, i)

        def Real(self):
            return self.getToken(CnsMRParser.Real, 0)

        def Error(self):
            return self.getToken(CnsMRParser.Error, 0)

        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def Phase(self):
            return self.getToken(CnsMRParser.Phase, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Rdc_potential(self):
            return self.getToken(CnsMRParser.Rdc_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def All(self):
            return self.getToken(CnsMRParser.All, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def Size(self):
            return self.getToken(CnsMRParser.Size, 0)

        def Dimensions(self):
            return self.getToken(CnsMRParser.Dimensions, 0)

        def Zero(self):
            return self.getToken(CnsMRParser.Zero, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_conformation_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterConformation_statement" ):
                listener.enterConformation_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitConformation_statement" ):
                listener.exitConformation_statement(self)




    def conformation_statement(self):

        localctx = CnsMRParser.Conformation_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 76, self.RULE_conformation_statement)
        self._la = 0 # Token type
        try:
            self.state = 782
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.End, CnsMRParser.Assign]:
                self.enterOuterAlt(localctx, 1)
                self.state = 709
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Assign:
                    self.state = 706
                    self.conf_assign()
                    self.state = 711
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [CnsMRParser.Classification]:
                self.enterOuterAlt(localctx, 2)
                self.state = 712
                self.match(CnsMRParser.Classification)
                self.state = 713
                self.match(CnsMRParser.Equ_op)
                self.state = 714
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.Compressed]:
                self.enterOuterAlt(localctx, 3)
                self.state = 715
                self.match(CnsMRParser.Compressed)
                pass
            elif token in [CnsMRParser.Expectation]:
                self.enterOuterAlt(localctx, 4)
                self.state = 716
                self.match(CnsMRParser.Expectation)
                self.state = 717
                self.match(CnsMRParser.Integer)
                self.state = 719
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,54,self._ctx)
                if la_ == 1:
                    self.state = 718
                    self.match(CnsMRParser.Integer)


                self.state = 722
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,55,self._ctx)
                if la_ == 1:
                    self.state = 721
                    self.match(CnsMRParser.Integer)


                self.state = 725
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Integer:
                    self.state = 724
                    self.match(CnsMRParser.Integer)


                self.state = 727
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.Error]:
                self.enterOuterAlt(localctx, 5)
                self.state = 728
                self.match(CnsMRParser.Error)
                self.state = 729
                self.match(CnsMRParser.Equ_op)
                self.state = 730
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.ForceConstant]:
                self.enterOuterAlt(localctx, 6)
                self.state = 731
                self.match(CnsMRParser.ForceConstant)
                self.state = 732
                self.match(CnsMRParser.Equ_op)
                self.state = 733
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.Nrestraints]:
                self.enterOuterAlt(localctx, 7)
                self.state = 734
                self.match(CnsMRParser.Nrestraints)
                self.state = 735
                self.match(CnsMRParser.Equ_op)
                self.state = 736
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Phase]:
                self.enterOuterAlt(localctx, 8)
                self.state = 737
                self.match(CnsMRParser.Phase)
                self.state = 738
                self.match(CnsMRParser.Integer)
                self.state = 739
                self.match(CnsMRParser.Integer)
                self.state = 740
                self.match(CnsMRParser.Integer)
                self.state = 744
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,57,self._ctx)
                if la_ == 1:
                    self.state = 741
                    self.match(CnsMRParser.Integer)
                    self.state = 742
                    self.match(CnsMRParser.Integer)
                    self.state = 743
                    self.match(CnsMRParser.Integer)


                self.state = 749
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,58,self._ctx)
                if la_ == 1:
                    self.state = 746
                    self.match(CnsMRParser.Integer)
                    self.state = 747
                    self.match(CnsMRParser.Integer)
                    self.state = 748
                    self.match(CnsMRParser.Integer)


                self.state = 754
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Integer:
                    self.state = 751
                    self.match(CnsMRParser.Integer)
                    self.state = 752
                    self.match(CnsMRParser.Integer)
                    self.state = 753
                    self.match(CnsMRParser.Integer)


                pass
            elif token in [CnsMRParser.Potential]:
                self.enterOuterAlt(localctx, 9)
                self.state = 756
                self.match(CnsMRParser.Potential)
                self.state = 757
                self.match(CnsMRParser.Equ_op)
                self.state = 758
                self.match(CnsMRParser.Rdc_potential)
                pass
            elif token in [CnsMRParser.Print]:
                self.enterOuterAlt(localctx, 10)
                self.state = 759
                self.match(CnsMRParser.Print)
                self.state = 760
                self.match(CnsMRParser.Threshold)
                self.state = 761
                self.match(CnsMRParser.Real)
                self.state = 766
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.All]:
                    self.state = 762
                    self.match(CnsMRParser.All)
                    pass
                elif token in [CnsMRParser.Classification]:
                    self.state = 763
                    self.match(CnsMRParser.Classification)
                    self.state = 764
                    self.match(CnsMRParser.Equ_op)
                    self.state = 765
                    self.match(CnsMRParser.Simple_name)
                    pass
                else:
                    raise NoViableAltException(self)

                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 11)
                self.state = 768
                self.match(CnsMRParser.Reset)
                pass
            elif token in [CnsMRParser.Size]:
                self.enterOuterAlt(localctx, 12)
                self.state = 769
                self.match(CnsMRParser.Size)
                self.state = 770
                self.match(CnsMRParser.Dimensions)
                self.state = 771
                self.match(CnsMRParser.Integer)
                self.state = 773
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,61,self._ctx)
                if la_ == 1:
                    self.state = 772
                    self.match(CnsMRParser.Integer)


                self.state = 776
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,62,self._ctx)
                if la_ == 1:
                    self.state = 775
                    self.match(CnsMRParser.Integer)


                self.state = 779
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Integer:
                    self.state = 778
                    self.match(CnsMRParser.Integer)


                pass
            elif token in [CnsMRParser.Zero]:
                self.enterOuterAlt(localctx, 13)
                self.state = 781
                self.match(CnsMRParser.Zero)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Conf_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_conf_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterConf_assign" ):
                listener.enterConf_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitConf_assign" ):
                listener.exitConf_assign(self)




    def conf_assign(self):

        localctx = CnsMRParser.Conf_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 78, self.RULE_conf_assign)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 784
            self.match(CnsMRParser.Assign)
            self.state = 785
            self.selection()
            self.state = 786
            self.selection()
            self.state = 787
            self.selection()
            self.state = 788
            self.selection()
            self.state = 794
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,65,self._ctx)
            if la_ == 1:
                self.state = 789
                self.selection()
                self.state = 790
                self.selection()
                self.state = 791
                self.selection()
                self.state = 792
                self.selection()


            self.state = 801
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,66,self._ctx)
            if la_ == 1:
                self.state = 796
                self.selection()
                self.state = 797
                self.selection()
                self.state = 798
                self.selection()
                self.state = 799
                self.selection()


            self.state = 808
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.L_paren:
                self.state = 803
                self.selection()
                self.state = 804
                self.selection()
                self.state = 805
                self.selection()
                self.state = 806
                self.selection()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Diffusion_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def dani_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Dani_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Dani_assignContext,i)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def Coefficients(self):
            return self.getToken(CnsMRParser.Coefficients, 0)

        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Rdc_potential(self):
            return self.getToken(CnsMRParser.Rdc_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_diffusion_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDiffusion_statement" ):
                listener.enterDiffusion_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDiffusion_statement" ):
                listener.exitDiffusion_statement(self)




    def diffusion_statement(self):

        localctx = CnsMRParser.Diffusion_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 80, self.RULE_diffusion_statement)
        self._la = 0 # Token type
        try:
            self.state = 838
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.End, CnsMRParser.Assign]:
                self.enterOuterAlt(localctx, 1)
                self.state = 813
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Assign:
                    self.state = 810
                    self.dani_assign()
                    self.state = 815
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [CnsMRParser.Classification]:
                self.enterOuterAlt(localctx, 2)
                self.state = 816
                self.match(CnsMRParser.Classification)
                self.state = 817
                self.match(CnsMRParser.Equ_op)
                self.state = 818
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.Coefficients]:
                self.enterOuterAlt(localctx, 3)
                self.state = 819
                self.match(CnsMRParser.Coefficients)
                self.state = 820
                self.match(CnsMRParser.Real)
                self.state = 821
                self.match(CnsMRParser.Real)
                self.state = 822
                self.match(CnsMRParser.Real)
                self.state = 823
                self.match(CnsMRParser.Real)
                self.state = 824
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.ForceConstant]:
                self.enterOuterAlt(localctx, 4)
                self.state = 825
                self.match(CnsMRParser.ForceConstant)
                self.state = 826
                self.match(CnsMRParser.Equ_op)
                self.state = 827
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.Nrestraints]:
                self.enterOuterAlt(localctx, 5)
                self.state = 828
                self.match(CnsMRParser.Nrestraints)
                self.state = 829
                self.match(CnsMRParser.Equ_op)
                self.state = 830
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Potential]:
                self.enterOuterAlt(localctx, 6)
                self.state = 831
                self.match(CnsMRParser.Potential)
                self.state = 832
                self.match(CnsMRParser.Equ_op)
                self.state = 833
                self.match(CnsMRParser.Rdc_potential)
                pass
            elif token in [CnsMRParser.Print]:
                self.enterOuterAlt(localctx, 7)
                self.state = 834
                self.match(CnsMRParser.Print)
                self.state = 835
                self.match(CnsMRParser.Threshold)
                self.state = 836
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 8)
                self.state = 837
                self.match(CnsMRParser.Reset)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dani_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def getRuleIndex(self):
            return CnsMRParser.RULE_dani_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDani_assign" ):
                listener.enterDani_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDani_assign" ):
                listener.exitDani_assign(self)




    def dani_assign(self):

        localctx = CnsMRParser.Dani_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 82, self.RULE_dani_assign)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 840
            self.match(CnsMRParser.Assign)
            self.state = 841
            self.selection()
            self.state = 842
            self.selection()
            self.state = 843
            self.selection()
            self.state = 844
            self.selection()
            self.state = 845
            self.selection()
            self.state = 846
            self.selection()
            self.state = 847
            self.match(CnsMRParser.Real)
            self.state = 848
            self.match(CnsMRParser.Real)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class One_bond_coupling_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def one_bond_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.One_bond_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.One_bond_assignContext,i)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def Coefficients(self):
            return self.getToken(CnsMRParser.Coefficients, 0)

        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Rdc_potential(self):
            return self.getToken(CnsMRParser.Rdc_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_one_bond_coupling_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterOne_bond_coupling_statement" ):
                listener.enterOne_bond_coupling_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitOne_bond_coupling_statement" ):
                listener.exitOne_bond_coupling_statement(self)




    def one_bond_coupling_statement(self):

        localctx = CnsMRParser.One_bond_coupling_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 84, self.RULE_one_bond_coupling_statement)
        self._la = 0 # Token type
        try:
            self.state = 880
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.End, CnsMRParser.Assign]:
                self.enterOuterAlt(localctx, 1)
                self.state = 853
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Assign:
                    self.state = 850
                    self.one_bond_assign()
                    self.state = 855
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [CnsMRParser.Classification]:
                self.enterOuterAlt(localctx, 2)
                self.state = 856
                self.match(CnsMRParser.Classification)
                self.state = 857
                self.match(CnsMRParser.Equ_op)
                self.state = 858
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.Coefficients]:
                self.enterOuterAlt(localctx, 3)
                self.state = 859
                self.match(CnsMRParser.Coefficients)
                self.state = 860
                self.match(CnsMRParser.Real)
                self.state = 861
                self.match(CnsMRParser.Real)
                self.state = 862
                self.match(CnsMRParser.Real)
                self.state = 863
                self.match(CnsMRParser.Real)
                self.state = 864
                self.match(CnsMRParser.Real)
                self.state = 865
                self.match(CnsMRParser.Real)
                self.state = 866
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.ForceConstant]:
                self.enterOuterAlt(localctx, 4)
                self.state = 867
                self.match(CnsMRParser.ForceConstant)
                self.state = 868
                self.match(CnsMRParser.Equ_op)
                self.state = 869
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.Nrestraints]:
                self.enterOuterAlt(localctx, 5)
                self.state = 870
                self.match(CnsMRParser.Nrestraints)
                self.state = 871
                self.match(CnsMRParser.Equ_op)
                self.state = 872
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Potential]:
                self.enterOuterAlt(localctx, 6)
                self.state = 873
                self.match(CnsMRParser.Potential)
                self.state = 874
                self.match(CnsMRParser.Equ_op)
                self.state = 875
                self.match(CnsMRParser.Rdc_potential)
                pass
            elif token in [CnsMRParser.Print]:
                self.enterOuterAlt(localctx, 7)
                self.state = 876
                self.match(CnsMRParser.Print)
                self.state = 877
                self.match(CnsMRParser.Threshold)
                self.state = 878
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 8)
                self.state = 879
                self.match(CnsMRParser.Reset)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class One_bond_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def getRuleIndex(self):
            return CnsMRParser.RULE_one_bond_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterOne_bond_assign" ):
                listener.enterOne_bond_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitOne_bond_assign" ):
                listener.exitOne_bond_assign(self)




    def one_bond_assign(self):

        localctx = CnsMRParser.One_bond_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 86, self.RULE_one_bond_assign)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 882
            self.match(CnsMRParser.Assign)
            self.state = 883
            self.selection()
            self.state = 884
            self.selection()
            self.state = 885
            self.selection()
            self.state = 886
            self.selection()
            self.state = 887
            self.selection()
            self.state = 888
            self.selection()
            self.state = 889
            self.selection()
            self.state = 890
            self.selection()
            self.state = 891
            self.match(CnsMRParser.Real)
            self.state = 892
            self.match(CnsMRParser.Real)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Angle_db_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def angle_db_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Angle_db_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Angle_db_assignContext,i)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def DerivFlag(self):
            return self.getToken(CnsMRParser.DerivFlag, 0)

        def Expectation(self):
            return self.getToken(CnsMRParser.Expectation, 0)

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Integer)
            else:
                return self.getToken(CnsMRParser.Integer, i)

        def Real(self):
            return self.getToken(CnsMRParser.Real, 0)

        def Error(self):
            return self.getToken(CnsMRParser.Error, 0)

        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Rdc_potential(self):
            return self.getToken(CnsMRParser.Rdc_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def All(self):
            return self.getToken(CnsMRParser.All, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def Size(self):
            return self.getToken(CnsMRParser.Size, 0)

        def Angle_dihedral(self):
            return self.getToken(CnsMRParser.Angle_dihedral, 0)

        def Zero(self):
            return self.getToken(CnsMRParser.Zero, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_angle_db_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAngle_db_statement" ):
                listener.enterAngle_db_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAngle_db_statement" ):
                listener.exitAngle_db_statement(self)




    def angle_db_statement(self):

        localctx = CnsMRParser.Angle_db_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 88, self.RULE_angle_db_statement)
        self._la = 0 # Token type
        try:
            self.state = 937
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.End, CnsMRParser.Assign]:
                self.enterOuterAlt(localctx, 1)
                self.state = 897
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Assign:
                    self.state = 894
                    self.angle_db_assign()
                    self.state = 899
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                pass
            elif token in [CnsMRParser.Classification]:
                self.enterOuterAlt(localctx, 2)
                self.state = 900
                self.match(CnsMRParser.Classification)
                self.state = 901
                self.match(CnsMRParser.Equ_op)
                self.state = 902
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.DerivFlag]:
                self.enterOuterAlt(localctx, 3)
                self.state = 903
                self.match(CnsMRParser.DerivFlag)
                self.state = 904
                self.match(CnsMRParser.Equ_op)
                self.state = 905
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.Expectation]:
                self.enterOuterAlt(localctx, 4)
                self.state = 906
                self.match(CnsMRParser.Expectation)
                self.state = 907
                self.match(CnsMRParser.Integer)
                self.state = 908
                self.match(CnsMRParser.Integer)
                self.state = 909
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.Error]:
                self.enterOuterAlt(localctx, 5)
                self.state = 910
                self.match(CnsMRParser.Error)
                self.state = 911
                self.match(CnsMRParser.Equ_op)
                self.state = 912
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.ForceConstant]:
                self.enterOuterAlt(localctx, 6)
                self.state = 913
                self.match(CnsMRParser.ForceConstant)
                self.state = 914
                self.match(CnsMRParser.Equ_op)
                self.state = 915
                self.match(CnsMRParser.Real)
                pass
            elif token in [CnsMRParser.Nrestraints]:
                self.enterOuterAlt(localctx, 7)
                self.state = 916
                self.match(CnsMRParser.Nrestraints)
                self.state = 917
                self.match(CnsMRParser.Equ_op)
                self.state = 918
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Potential]:
                self.enterOuterAlt(localctx, 8)
                self.state = 919
                self.match(CnsMRParser.Potential)
                self.state = 920
                self.match(CnsMRParser.Equ_op)
                self.state = 921
                self.match(CnsMRParser.Rdc_potential)
                pass
            elif token in [CnsMRParser.Print]:
                self.enterOuterAlt(localctx, 9)
                self.state = 922
                self.match(CnsMRParser.Print)
                self.state = 923
                self.match(CnsMRParser.Threshold)
                self.state = 924
                self.match(CnsMRParser.Real)
                self.state = 929
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.All]:
                    self.state = 925
                    self.match(CnsMRParser.All)
                    pass
                elif token in [CnsMRParser.Classification]:
                    self.state = 926
                    self.match(CnsMRParser.Classification)
                    self.state = 927
                    self.match(CnsMRParser.Equ_op)
                    self.state = 928
                    self.match(CnsMRParser.Simple_name)
                    pass
                else:
                    raise NoViableAltException(self)

                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 10)
                self.state = 931
                self.match(CnsMRParser.Reset)
                pass
            elif token in [CnsMRParser.Size]:
                self.enterOuterAlt(localctx, 11)
                self.state = 932
                self.match(CnsMRParser.Size)
                self.state = 933
                self.match(CnsMRParser.Angle_dihedral)
                self.state = 934
                self.match(CnsMRParser.Integer)
                self.state = 935
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Zero]:
                self.enterOuterAlt(localctx, 12)
                self.state = 936
                self.match(CnsMRParser.Zero)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Angle_db_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_angle_db_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAngle_db_assign" ):
                listener.enterAngle_db_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAngle_db_assign" ):
                listener.exitAngle_db_assign(self)




    def angle_db_assign(self):

        localctx = CnsMRParser.Angle_db_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 90, self.RULE_angle_db_assign)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 939
            self.match(CnsMRParser.Assign)
            self.state = 940
            self.selection()
            self.state = 941
            self.selection()
            self.state = 942
            self.selection()
            self.state = 943
            self.selection()
            self.state = 944
            self.selection()
            self.state = 945
            self.selection()
            self.state = 946
            self.selection()
            self.state = 947
            self.selection()
            self.state = 948
            self.selection()
            self.state = 949
            self.selection()
            self.state = 950
            self.selection()
            self.state = 952
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.L_paren:
                self.state = 951
                self.selection()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SelectionContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def L_paren(self):
            return self.getToken(CnsMRParser.L_paren, 0)

        def selection_expression(self):
            return self.getTypedRuleContext(CnsMRParser.Selection_expressionContext,0)


        def R_paren(self):
            return self.getToken(CnsMRParser.R_paren, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_selection

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSelection" ):
                listener.enterSelection(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSelection" ):
                listener.exitSelection(self)




    def selection(self):

        localctx = CnsMRParser.SelectionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 92, self.RULE_selection)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 954
            self.match(CnsMRParser.L_paren)
            self.state = 955
            self.selection_expression()
            self.state = 956
            self.match(CnsMRParser.R_paren)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Selection_expressionContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def term(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.TermContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.TermContext,i)


        def Or_op(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Or_op)
            else:
                return self.getToken(CnsMRParser.Or_op, i)

        def getRuleIndex(self):
            return CnsMRParser.RULE_selection_expression

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSelection_expression" ):
                listener.enterSelection_expression(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSelection_expression" ):
                listener.exitSelection_expression(self)




    def selection_expression(self):

        localctx = CnsMRParser.Selection_expressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 94, self.RULE_selection_expression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 958
            self.term()
            self.state = 963
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==CnsMRParser.Or_op:
                self.state = 959
                self.match(CnsMRParser.Or_op)
                self.state = 960
                self.term()
                self.state = 965
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TermContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def factor(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.FactorContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.FactorContext,i)


        def And_op(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.And_op)
            else:
                return self.getToken(CnsMRParser.And_op, i)

        def getRuleIndex(self):
            return CnsMRParser.RULE_term

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTerm" ):
                listener.enterTerm(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTerm" ):
                listener.exitTerm(self)




    def term(self):

        localctx = CnsMRParser.TermContext(self, self._ctx, self.state)
        self.enterRule(localctx, 96, self.RULE_term)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 966
            self.factor(0)
            self.state = 971
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==CnsMRParser.And_op:
                self.state = 967
                self.match(CnsMRParser.And_op)
                self.state = 968
                self.factor(0)
                self.state = 973
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FactorContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def L_paren(self):
            return self.getToken(CnsMRParser.L_paren, 0)

        def selection_expression(self):
            return self.getTypedRuleContext(CnsMRParser.Selection_expressionContext,0)


        def R_paren(self):
            return self.getToken(CnsMRParser.R_paren, 0)

        def All(self):
            return self.getToken(CnsMRParser.All, 0)

        def Atom(self):
            return self.getToken(CnsMRParser.Atom, 0)

        def Simple_names(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Simple_names)
            else:
                return self.getToken(CnsMRParser.Simple_names, i)

        def Simple_name(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Simple_name)
            else:
                return self.getToken(CnsMRParser.Simple_name, i)

        def Integers(self):
            return self.getToken(CnsMRParser.Integers, 0)

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Integer)
            else:
                return self.getToken(CnsMRParser.Integer, i)

        def Attribute(self):
            return self.getToken(CnsMRParser.Attribute, 0)

        def Attr_properties(self):
            return self.getToken(CnsMRParser.Attr_properties, 0)

        def Comparison_ops(self):
            return self.getToken(CnsMRParser.Comparison_ops, 0)

        def Real(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Real)
            else:
                return self.getToken(CnsMRParser.Real, i)

        def Abs(self):
            return self.getToken(CnsMRParser.Abs, 0)

        def BondedTo(self):
            return self.getToken(CnsMRParser.BondedTo, 0)

        def factor(self):
            return self.getTypedRuleContext(CnsMRParser.FactorContext,0)


        def ByGroup(self):
            return self.getToken(CnsMRParser.ByGroup, 0)

        def ByRes(self):
            return self.getToken(CnsMRParser.ByRes, 0)

        def Chemical(self):
            return self.getToken(CnsMRParser.Chemical, 0)

        def Colon(self):
            return self.getToken(CnsMRParser.Colon, 0)

        def Fbox(self):
            return self.getToken(CnsMRParser.Fbox, 0)

        def Hydrogen(self):
            return self.getToken(CnsMRParser.Hydrogen, 0)

        def Id(self):
            return self.getToken(CnsMRParser.Id, 0)

        def Known(self):
            return self.getToken(CnsMRParser.Known, 0)

        def Name(self):
            return self.getToken(CnsMRParser.Name, 0)

        def NONE(self):
            return self.getToken(CnsMRParser.NONE, 0)

        def Not_op(self):
            return self.getToken(CnsMRParser.Not_op, 0)

        def Point(self):
            return self.getToken(CnsMRParser.Point, 0)

        def Cut(self):
            return self.getToken(CnsMRParser.Cut, 0)

        def Comma(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Comma)
            else:
                return self.getToken(CnsMRParser.Comma, i)

        def Tail(self):
            return self.getToken(CnsMRParser.Tail, 0)

        def Equ_op(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Equ_op)
            else:
                return self.getToken(CnsMRParser.Equ_op, i)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def Head(self):
            return self.getToken(CnsMRParser.Head, 0)

        def Previous(self):
            return self.getToken(CnsMRParser.Previous, 0)

        def Pseudo(self):
            return self.getToken(CnsMRParser.Pseudo, 0)

        def Residue(self):
            return self.getToken(CnsMRParser.Residue, 0)

        def Resname(self):
            return self.getToken(CnsMRParser.Resname, 0)

        def SegIdentifier(self):
            return self.getToken(CnsMRParser.SegIdentifier, 0)

        def Double_quote_string(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Double_quote_string)
            else:
                return self.getToken(CnsMRParser.Double_quote_string, i)

        def Sfbox(self):
            return self.getToken(CnsMRParser.Sfbox, 0)

        def Store_1(self):
            return self.getToken(CnsMRParser.Store_1, 0)

        def Store_2(self):
            return self.getToken(CnsMRParser.Store_2, 0)

        def Store_3(self):
            return self.getToken(CnsMRParser.Store_3, 0)

        def Store_4(self):
            return self.getToken(CnsMRParser.Store_4, 0)

        def Store_5(self):
            return self.getToken(CnsMRParser.Store_5, 0)

        def Store_6(self):
            return self.getToken(CnsMRParser.Store_6, 0)

        def Store_7(self):
            return self.getToken(CnsMRParser.Store_7, 0)

        def Store_8(self):
            return self.getToken(CnsMRParser.Store_8, 0)

        def Store_9(self):
            return self.getToken(CnsMRParser.Store_9, 0)

        def Tag(self):
            return self.getToken(CnsMRParser.Tag, 0)

        def Around(self):
            return self.getToken(CnsMRParser.Around, 0)

        def Saround(self):
            return self.getToken(CnsMRParser.Saround, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_factor

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFactor" ):
                listener.enterFactor(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFactor" ):
                listener.exitFactor(self)



    def factor(self, _p:int=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = CnsMRParser.FactorContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 98
        self.enterRecursionRule(localctx, 98, self.RULE_factor, _p)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1111
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,94,self._ctx)
            if la_ == 1:
                self.state = 975
                self.match(CnsMRParser.L_paren)
                self.state = 976
                self.selection_expression()
                self.state = 977
                self.match(CnsMRParser.R_paren)
                pass

            elif la_ == 2:
                self.state = 979
                self.match(CnsMRParser.All)
                pass

            elif la_ == 3:
                self.state = 980
                self.match(CnsMRParser.Atom)
                self.state = 981
                _la = self._input.LA(1)
                if not(_la==CnsMRParser.Simple_name or _la==CnsMRParser.Simple_names):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 982
                _la = self._input.LA(1)
                if not(_la==CnsMRParser.Integer or _la==CnsMRParser.Integers):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 983
                _la = self._input.LA(1)
                if not(_la==CnsMRParser.Simple_name or _la==CnsMRParser.Simple_names):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                pass

            elif la_ == 4:
                self.state = 984
                self.match(CnsMRParser.Attribute)
                self.state = 986
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Abs:
                    self.state = 985
                    self.match(CnsMRParser.Abs)


                self.state = 988
                self.match(CnsMRParser.Attr_properties)
                self.state = 989
                self.match(CnsMRParser.Comparison_ops)
                self.state = 990
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 5:
                self.state = 991
                self.match(CnsMRParser.BondedTo)
                self.state = 992
                self.factor(30)
                pass

            elif la_ == 6:
                self.state = 993
                self.match(CnsMRParser.ByGroup)
                self.state = 994
                self.factor(29)
                pass

            elif la_ == 7:
                self.state = 995
                self.match(CnsMRParser.ByRes)
                self.state = 996
                self.factor(28)
                pass

            elif la_ == 8:
                self.state = 997
                self.match(CnsMRParser.Chemical)
                self.state = 1004
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.Simple_names]:
                    self.state = 998
                    self.match(CnsMRParser.Simple_names)
                    pass
                elif token in [CnsMRParser.Simple_name]:
                    self.state = 999
                    self.match(CnsMRParser.Simple_name)
                    self.state = 1002
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,79,self._ctx)
                    if la_ == 1:
                        self.state = 1000
                        self.match(CnsMRParser.Colon)
                        self.state = 1001
                        self.match(CnsMRParser.Simple_name)


                    pass
                else:
                    raise NoViableAltException(self)

                pass

            elif la_ == 9:
                self.state = 1006
                self.match(CnsMRParser.Fbox)
                self.state = 1007
                self.match(CnsMRParser.Real)
                self.state = 1008
                self.match(CnsMRParser.Real)
                self.state = 1009
                self.match(CnsMRParser.Real)
                self.state = 1010
                self.match(CnsMRParser.Real)
                self.state = 1011
                self.match(CnsMRParser.Real)
                self.state = 1012
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 10:
                self.state = 1013
                self.match(CnsMRParser.Hydrogen)
                pass

            elif la_ == 11:
                self.state = 1014
                self.match(CnsMRParser.Id)
                self.state = 1015
                self.match(CnsMRParser.Integer)
                pass

            elif la_ == 12:
                self.state = 1016
                self.match(CnsMRParser.Known)
                pass

            elif la_ == 13:
                self.state = 1017
                self.match(CnsMRParser.Name)
                self.state = 1024
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.Simple_names]:
                    self.state = 1018
                    self.match(CnsMRParser.Simple_names)
                    pass
                elif token in [CnsMRParser.Simple_name]:
                    self.state = 1019
                    self.match(CnsMRParser.Simple_name)
                    self.state = 1022
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,81,self._ctx)
                    if la_ == 1:
                        self.state = 1020
                        self.match(CnsMRParser.Colon)
                        self.state = 1021
                        self.match(CnsMRParser.Simple_name)


                    pass
                else:
                    raise NoViableAltException(self)

                pass

            elif la_ == 14:
                self.state = 1026
                self.match(CnsMRParser.NONE)
                pass

            elif la_ == 15:
                self.state = 1027
                self.match(CnsMRParser.Not_op)
                self.state = 1028
                self.factor(20)
                pass

            elif la_ == 16:
                self.state = 1029
                self.match(CnsMRParser.Point)
                self.state = 1030
                self.match(CnsMRParser.L_paren)
                self.state = 1031
                self.match(CnsMRParser.Real)
                self.state = 1033
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Comma:
                    self.state = 1032
                    self.match(CnsMRParser.Comma)


                self.state = 1035
                self.match(CnsMRParser.Real)
                self.state = 1037
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Comma:
                    self.state = 1036
                    self.match(CnsMRParser.Comma)


                self.state = 1039
                self.match(CnsMRParser.Real)
                self.state = 1040
                self.match(CnsMRParser.R_paren)
                self.state = 1041
                self.match(CnsMRParser.Cut)
                self.state = 1042
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 17:
                self.state = 1043
                self.match(CnsMRParser.Point)
                self.state = 1044
                self.match(CnsMRParser.L_paren)
                self.state = 1045
                self.match(CnsMRParser.Tail)
                self.state = 1046
                self.match(CnsMRParser.Equ_op)
                self.state = 1047
                self.selection()
                self.state = 1049
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Comma:
                    self.state = 1048
                    self.match(CnsMRParser.Comma)


                self.state = 1054
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Head:
                    self.state = 1051
                    self.match(CnsMRParser.Head)
                    self.state = 1052
                    self.match(CnsMRParser.Equ_op)
                    self.state = 1053
                    self.selection()


                self.state = 1056
                self.match(CnsMRParser.R_paren)
                self.state = 1057
                self.match(CnsMRParser.Cut)
                self.state = 1058
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 18:
                self.state = 1060
                self.match(CnsMRParser.Previous)
                pass

            elif la_ == 19:
                self.state = 1061
                self.match(CnsMRParser.Pseudo)
                pass

            elif la_ == 20:
                self.state = 1062
                self.match(CnsMRParser.Residue)
                self.state = 1069
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.Integers]:
                    self.state = 1063
                    self.match(CnsMRParser.Integers)
                    pass
                elif token in [CnsMRParser.Integer]:
                    self.state = 1064
                    self.match(CnsMRParser.Integer)
                    self.state = 1067
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,87,self._ctx)
                    if la_ == 1:
                        self.state = 1065
                        self.match(CnsMRParser.Colon)
                        self.state = 1066
                        self.match(CnsMRParser.Integer)


                    pass
                else:
                    raise NoViableAltException(self)

                pass

            elif la_ == 21:
                self.state = 1071
                self.match(CnsMRParser.Resname)
                self.state = 1078
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.Simple_names]:
                    self.state = 1072
                    self.match(CnsMRParser.Simple_names)
                    pass
                elif token in [CnsMRParser.Simple_name]:
                    self.state = 1073
                    self.match(CnsMRParser.Simple_name)
                    self.state = 1076
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,89,self._ctx)
                    if la_ == 1:
                        self.state = 1074
                        self.match(CnsMRParser.Colon)
                        self.state = 1075
                        self.match(CnsMRParser.Simple_name)


                    pass
                else:
                    raise NoViableAltException(self)

                pass

            elif la_ == 22:
                self.state = 1080
                self.match(CnsMRParser.SegIdentifier)
                self.state = 1092
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.Simple_names]:
                    self.state = 1081
                    self.match(CnsMRParser.Simple_names)
                    pass
                elif token in [CnsMRParser.Simple_name]:
                    self.state = 1082
                    self.match(CnsMRParser.Simple_name)
                    self.state = 1085
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,91,self._ctx)
                    if la_ == 1:
                        self.state = 1083
                        self.match(CnsMRParser.Colon)
                        self.state = 1084
                        self.match(CnsMRParser.Simple_name)


                    pass
                elif token in [CnsMRParser.Double_quote_string]:
                    self.state = 1087
                    self.match(CnsMRParser.Double_quote_string)
                    self.state = 1090
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,92,self._ctx)
                    if la_ == 1:
                        self.state = 1088
                        self.match(CnsMRParser.Colon)
                        self.state = 1089
                        self.match(CnsMRParser.Double_quote_string)


                    pass
                else:
                    raise NoViableAltException(self)

                pass

            elif la_ == 23:
                self.state = 1094
                self.match(CnsMRParser.Sfbox)
                self.state = 1095
                self.match(CnsMRParser.Real)
                self.state = 1096
                self.match(CnsMRParser.Real)
                self.state = 1097
                self.match(CnsMRParser.Real)
                self.state = 1098
                self.match(CnsMRParser.Real)
                self.state = 1099
                self.match(CnsMRParser.Real)
                self.state = 1100
                self.match(CnsMRParser.Real)
                pass

            elif la_ == 24:
                self.state = 1101
                self.match(CnsMRParser.Store_1)
                pass

            elif la_ == 25:
                self.state = 1102
                self.match(CnsMRParser.Store_2)
                pass

            elif la_ == 26:
                self.state = 1103
                self.match(CnsMRParser.Store_3)
                pass

            elif la_ == 27:
                self.state = 1104
                self.match(CnsMRParser.Store_4)
                pass

            elif la_ == 28:
                self.state = 1105
                self.match(CnsMRParser.Store_5)
                pass

            elif la_ == 29:
                self.state = 1106
                self.match(CnsMRParser.Store_6)
                pass

            elif la_ == 30:
                self.state = 1107
                self.match(CnsMRParser.Store_7)
                pass

            elif la_ == 31:
                self.state = 1108
                self.match(CnsMRParser.Store_8)
                pass

            elif la_ == 32:
                self.state = 1109
                self.match(CnsMRParser.Store_9)
                pass

            elif la_ == 33:
                self.state = 1110
                self.match(CnsMRParser.Tag)
                pass


            self._ctx.stop = self._input.LT(-1)
            self.state = 1121
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,96,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    self.state = 1119
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,95,self._ctx)
                    if la_ == 1:
                        localctx = CnsMRParser.FactorContext(self, _parentctx, _parentState)
                        self.pushNewRecursionContext(localctx, _startState, self.RULE_factor)
                        self.state = 1113
                        if not self.precpred(self._ctx, 33):
                            from antlr4.error.Errors import FailedPredicateException
                            raise FailedPredicateException(self, "self.precpred(self._ctx, 33)")
                        self.state = 1114
                        self.match(CnsMRParser.Around)
                        self.state = 1115
                        self.match(CnsMRParser.Real)
                        pass

                    elif la_ == 2:
                        localctx = CnsMRParser.FactorContext(self, _parentctx, _parentState)
                        self.pushNewRecursionContext(localctx, _startState, self.RULE_factor)
                        self.state = 1116
                        if not self.precpred(self._ctx, 13):
                            from antlr4.error.Errors import FailedPredicateException
                            raise FailedPredicateException(self, "self.precpred(self._ctx, 13)")
                        self.state = 1117
                        self.match(CnsMRParser.Saround)
                        self.state = 1118
                        self.match(CnsMRParser.Real)
                        pass

             
                self.state = 1123
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,96,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx



    def sempred(self, localctx:RuleContext, ruleIndex:int, predIndex:int):
        if self._predicates == None:
            self._predicates = dict()
        self._predicates[49] = self.factor_sempred
        pred = self._predicates.get(ruleIndex, None)
        if pred is None:
            raise Exception("No predicate with index:" + str(ruleIndex))
        else:
            return pred(localctx, predIndex)

    def factor_sempred(self, localctx:FactorContext, predIndex:int):
            if predIndex == 0:
                return self.precpred(self._ctx, 33)
         

            if predIndex == 1:
                return self.precpred(self._ctx, 13)
         




